/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or bbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * bbbuild/licenses/cddl-gpl-2-cp.txt.  This particular file is subject to
 * the "Classpath" exception as provided in the GPL Version 2 section of the
 * License file that accompanied this code. If applicable, add the following
 * below the License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 * Portions Copyrighted 2017 George Waldon
 */
package org.biobeans.modules.biocore.actions.list;

import java.io.PrintWriter;
import java.text.DateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.logging.Logger;
import org.biobeans.modules.biocore.actions.SequenceProviderAction;
import org.biobeans.modules.biocore.sequences.RichIdentifier;
import org.biojava.bio.seq.Sequence;
import org.openide.nodes.Node;
import org.openide.util.NbBundle;
import static org.openide.util.actions.CookieAction.MODE_ANY;
import org.openide.windows.IOProvider;
import org.openide.windows.InputOutput;


public abstract class ListRawSequenceAction extends SequenceProviderAction {

    private static final Logger LOGGER = Logger.getLogger(ListRawSequenceAction.class.getName());

    @Override
    abstract protected int getSequenceFormat();
    
    @Override
    public String getName() {
        return NbBundle.getMessage(ListRawSequenceAction.class, "LBL_ListRawSeq");
    }    
 
    @Override
    protected int mode() {
        return MODE_ANY;
    }
    
    @Override
    protected int getMinLength() {
        return 1;
    }

    @Override
    protected void performSeqAction(Node node, Sequence[] seqarray, boolean comp) {
        String name = windowTitle(comp == false ? node.getName() : seqarray[0].getName());

        InputOutput io = IOProvider.getDefault().getIO(name, false);
        io.select();
        io.setFocusTaken(false);

        PrintWriter pw = io.getOut();
        try {
            pw.println(Header());
            pw.println ("");
            
            for (Sequence seqarray1 : seqarray) {
                int slen = seqarray1.length();
                if (!isSupported(seqarray1) || slen==0) {
                    continue;
                }
                RichIdentifier rid = RichIdentifier.create(seqarray1);
                String nname = rid.getDisplayName();
                pw.println (nname);
                int step = 80;
                int seqstart = 1;
                while(seqstart<slen) {
                    int seqend = seqstart+step-1;
                    if(seqend>slen) {
                        seqend = slen;
                    }
                    String s = seqarray1.subStr(seqstart, seqend);
                    pw.println (s);
                    seqstart += step;
                }
                pw.println ("");
            }
        } finally {
            pw.close();
        }
    }
    
    public static String Header() {
        String head =  "# BioBeans " /*+ version*/;
        Calendar cal = new GregorianCalendar();
        DateFormat date = DateFormat.getDateInstance(DateFormat.FULL);
        DateFormat time = DateFormat.getTimeInstance(DateFormat.FULL);
        String dat = date.format(cal.getTime());
        String tim = time.format(cal.getTime());
        return head + "  -  " + dat + "   " + tim;
    }
}
