/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or bbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * bbbuild/licenses/cddl-gpl-2-cp.txt.  This particular file is subject to
 * the "Classpath" exception as provided in the GPL Version 2 section of the
 * License file that accompanied this code. If applicable, add the following
 * below the License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 * Portions Copyrighted 2017 George Waldon
 */

package org.biobeans.modules.biocore.seqfiles.gck;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.DataInput;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.io.Reader;
import java.util.logging.Logger;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.io.ParseException;
import org.biojava.bio.seq.io.SeqIOListener;
import org.biojava.bio.seq.io.SymbolTokenization;
import org.biojava.bio.symbol.IllegalAlphabetException;
import org.biojava.bio.symbol.IllegalSymbolException;
import org.biojava.bio.symbol.SimpleSymbolList;
import org.biojava.bio.symbol.Symbol;
import org.biojava.bio.symbol.SymbolList;
import org.biojavax.Namespace;
import org.biojavax.RichObjectFactory;
import org.biojavax.bio.seq.io.RichSeqIOListener;
import org.biojavax.bio.seq.io.RichSequenceFormat;
import org.biojavax.bio.taxa.NCBITaxon;

/**
 *
 * @author gwaldon
 */
public class GCKFormat extends RichSequenceFormat.HeaderlessFormat {

    private static final Logger LOGGER = Logger.getLogger(GCKFormat.class.getName());
    
    @Override
    public boolean canRead(BufferedInputStream stream) throws IOException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public SymbolTokenization guessSymbolTokenization(BufferedInputStream stream) throws IOException {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    public static class DataReader extends BufferedReader implements DataInput {
        DataInputStream dis;
        
        public DataReader(Reader reader,DataInputStream dis) {
            super(reader);
            this.dis = dis;
        }

        @Override
        public void readFully(byte[] b) throws IOException {
            dis.readFully(b);
        }

        @Override
        public void readFully(byte[] b, int off, int len) throws IOException {
            dis.readFully(b, off, len);
        }

        @Override
        public int skipBytes(int n) throws IOException {
            return dis.skipBytes(n);
        }

        @Override
        public boolean readBoolean() throws IOException {
            return dis.readBoolean();
        }

        @Override
        public byte readByte() throws IOException {
            return dis.readByte();
        }

        @Override
        public int readUnsignedByte() throws IOException {
            return dis.readUnsignedByte();
        }

        @Override
        public short readShort() throws IOException {
            return dis.readShort();
        }

        @Override
        public int readUnsignedShort() throws IOException {
            return dis.readUnsignedShort();
        }

        @Override
        public char readChar() throws IOException {
            return dis.readChar();
        }

        @Override
        public int readInt() throws IOException {
            return dis.readInt();
        }

        @Override
        public long readLong() throws IOException {
            return dis.readLong();
        }

        @Override
        public float readFloat() throws IOException {
            return dis.readFloat();
        }

        @Override
        public double readDouble() throws IOException {
            return dis.readDouble();
        }

        @Override
        public String readUTF() throws IOException {
            return dis.readUTF();
        }
    }

    private NCBITaxon tax = null;
    private String name = "";
    private String organism = "";
    private String accession = "";
    private String identifier = "";
    
    void setName(String newname) {
        name = newname;
    }
    
    @Override
    public boolean readRichSequence(BufferedReader breader, 
            SymbolTokenization symParser, RichSeqIOListener rlistener, Namespace ns) 
            throws BioException, IllegalSymbolException, IOException {
        
        DataInput reader = (DataInput) breader;
        rlistener.startSequence();
        if (ns==null) ns=RichObjectFactory.getDefaultNamespace();
        rlistener.setNamespace(ns);
        
        int value = 0;
        
        //GCK Header
        value = reader.readInt(); //Format version ?
        for(int i=0;i<5;i++)
            reader.readInt();
        value = reader.readInt(); //Length of sequence
        
        //Read sequence to follow
        StringBuilder buf = new StringBuilder(value);
        buf.setLength(value);
        for(int i=0;i<value;i++) {
            char ch = (char) reader.readUnsignedByte();
            buf.setCharAt(i,ch);
        }
        try {
            SymbolList sl = new SimpleSymbolList(symParser,
                    buf.toString().replaceAll("\\s+", "").replaceAll("[\\.|~]", "-"));
            rlistener.addSymbols(symParser.getAlphabet(),
                    sl.toList().toArray(new Symbol[0]),
                    0, sl.length());
        } catch (IllegalAlphabetException e) {
            String message = ParseException.newMessage(this.getClass(), accession, identifier, "Bad sequence section", "");
            throw new ParseException(e, message);
        }
                
        rlistener.setName(name);
        rlistener.setAccession(accession);
        rlistener.endSequence();
        return false;
    }

    @Override
    public void writeSequence(Sequence seq, Namespace ns) throws IOException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public boolean readSequence(BufferedReader reader, SymbolTokenization symParser, SeqIOListener listener) throws BioException, IllegalSymbolException, IOException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void writeSequence(Sequence seq, PrintStream os) throws IOException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Deprecated
    @Override
    public void writeSequence(Sequence seq, String format, PrintStream os) throws IOException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Deprecated
    @Override
    public String getDefaultFormat() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

}
