/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or bbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * bbbuild/licenses/cddl-gpl-2-cp.txt.  This particular file is subject to
 * the "Classpath" exception as provided in the GPL Version 2 section of the
 * License file that accompanied this code. If applicable, add the following
 * below the License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 * Portions Copyrighted 2017 George Waldon
 */


package org.biobeans.modules.biocore.seqfiles.raw;
import java.io.IOException;
import javax.swing.JEditorPane;
import org.biobeans.modules.biocore.sequences.MIMEType;
import org.openide.cookies.*;
import org.openide.filesystems.FileLock;
import org.openide.filesystems.FileObject;
import org.openide.nodes.Node.Cookie;
import org.openide.text.CloneableEditor;
import org.openide.text.DataEditorSupport;
import org.openide.util.UserCancelException;
import org.openide.windows.CloneableOpenSupport;



/** 
 * Editor support for raw sequence objects.
 *
 * @author gwaldon
 * 
 */
public final class RawEditorSupport extends DataEditorSupport implements OpenCookie, EditCookie, EditorCookie.Observable, PrintCookie {

    private final SaveCookie saveCookie = new SaveCookie() {
        @Override
        public void save() throws IOException {
            try {
                saveDocument();
            }catch(UserCancelException uce) {
            }
        }
    };
   
    RawEditorSupport(RawDataObject obj) {
        super(obj, new Environment(obj));
        setMIMEType(MIMEType.RAW);
    }

    @Override
    protected boolean asynchronousOpen() {
        return false;
    }

    @Override
    public void saveDocument() throws IOException {
        super.saveDocument();
        RawEditorSupport.this.getDataObject().setModified(false);
    }

    @Override
    protected boolean notifyModified() {
        if (!super.notifyModified()) 
            return false;
        addSaveCookie();
        return true;
    }

    @Override
    protected void notifyUnmodified() {
        super.notifyUnmodified();
        removeSaveCookie();
    }

    /** Helper method. Adds save cookie to the data object. */
    private void addSaveCookie() {
        RawDataObject obj = (RawDataObject)getDataObject();
        if(obj.getCookie(SaveCookie.class) == null) {
            obj.getCookieSet0().add(saveCookie);
            obj.setModified(true);
        }
    }

    private void removeSaveCookie() {
        RawDataObject obj = (RawDataObject)getDataObject();
        Cookie cookie = obj.getCookie(SaveCookie.class);
        if(cookie != null && cookie.equals(saveCookie)) {
            obj.getCookieSet0().remove(saveCookie);
            obj.setModified(false);
        }
    }

    private static class Environment extends DataEditorSupport.Env {

        private static final long serialVersionUID = 1L;
        
        public Environment(RawDataObject obj) {
            super(obj);
        }

        @Override
        protected FileObject getFile() {
            return getDataObject().getPrimaryFile();
        }

        @Override
        protected FileLock takeLock() throws IOException {
            return ((RawDataObject)getDataObject()).getPrimaryEntry().takeLock();
        }

        @Override
        public CloneableOpenSupport findCloneableOpenSupport() {
            return getDataObject().getCookie(RawEditorSupport.class);
        }
    } 

    
    
    @Override
    protected CloneableEditor createCloneableEditor() {
        return new RawEditor(this);
    }
    
    public static class RawEditor extends CloneableEditor {
        
        public RawEditor() {
        }
        
        public RawEditor(RawEditorSupport s) {
            super(s);
        }
        
        @Override
        public void requestActive() {
            super.requestActive();
            JEditorPane p = getEditorPane();
            if (p != null) {
                p.requestFocus();
            }
        }
        
        @Override
        protected void componentActivated() {
            super.componentActivated();
            JEditorPane p = getEditorPane();
            // give focus to the editor pane
            if (p != null) {
                p.requestFocusInWindow();
            }
        }
    }
    
}