/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or bbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * bbbuild/licenses/cddl-gpl-2-cp.txt.  This particular file is subject to
 * the "Classpath" exception as provided in the GPL Version 2 section of the
 * License file that accompanied this code. If applicable, add the following
 * below the License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 * Portions Copyrighted 2017 George Waldon
 */

package org.biobeans.modules.biocore.seqfiles.raw;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintStream;
import java.util.logging.Logger;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.io.ParseException;
import org.biojava.bio.seq.io.SeqIOListener;
import org.biojava.bio.seq.io.SymbolTokenization;
import org.biojava.bio.symbol.IllegalAlphabetException;
import org.biojava.bio.symbol.IllegalSymbolException;
import org.biojava.bio.symbol.SimpleSymbolList;
import org.biojava.bio.symbol.Symbol;
import org.biojava.bio.symbol.SymbolList;
import org.biojavax.Namespace;
import org.biojavax.RichObjectFactory;
import org.biojavax.bio.seq.io.RichSeqIOListener;
import org.biojavax.bio.seq.io.RichSequenceFormat;
import org.biojavax.bio.taxa.NCBITaxon;

/**
 *
 * @author gwaldon
 */
public class RawFormat extends RichSequenceFormat.HeaderlessFormat {

    private static final Logger LOGGER = Logger.getLogger(RawFormat.class.getName());
    
    @Override
    public boolean canRead(BufferedInputStream stream) throws IOException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public SymbolTokenization guessSymbolTokenization(BufferedInputStream stream) throws IOException {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    private NCBITaxon tax = null;
    private String name = "";
    private String organism = "";
    private String accession = "";
    private String identifier = "";
    
    void setName(String newname) {
        name = newname;
    }
    
    @Override
    public boolean readRichSequence(BufferedReader reader, 
            SymbolTokenization symParser, RichSeqIOListener rlistener, Namespace ns) 
            throws BioException, IllegalSymbolException, IOException {
        
        rlistener.startSequence();
        if (ns==null) ns=RichObjectFactory.getDefaultNamespace();
        rlistener.setNamespace(ns);
        
        StringBuilder buf = new StringBuilder();
        
        while(true) {
            int i = reader.read();
            if(i==-1) 
                break;
            char c = (char)i;
            if(Character.isDigit(c)) 
                continue;
            if(Character.isWhitespace(c)) 
                continue;
            if(c=='.'||c=='~')
                c = '-';
            buf.append(c);
        }

        try {
            SymbolList sl = new SimpleSymbolList(symParser,buf.toString());
            rlistener.addSymbols(symParser.getAlphabet(),
                    sl.toList().toArray(new Symbol[0]),
                    0, sl.length());
        } catch (IllegalAlphabetException e) {
            String message = ParseException.newMessage(this.getClass(), accession, identifier, "Bad sequence section", "");
            throw new ParseException(e, message);
        }
                
        rlistener.setName(name);
        rlistener.setAccession(accession);
        rlistener.endSequence();
        return false;
    }

    @Override
    public void writeSequence(Sequence seq, Namespace ns) throws IOException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public boolean readSequence(BufferedReader reader, SymbolTokenization symParser, SeqIOListener listener) throws BioException, IllegalSymbolException, IOException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void writeSequence(Sequence seq, PrintStream os) throws IOException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Deprecated
    @Override
    public void writeSequence(Sequence seq, String format, PrintStream os) throws IOException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Deprecated
    @Override
    public String getDefaultFormat() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

}
