/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or bbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * bbbuild/licenses/cddl-gpl-2-cp.txt.  This particular file is subject to
 * the "Classpath" exception as provided in the GPL Version 2 section of the
 * License file that accompanied this code. If applicable, add the following
 * below the License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 * Portions Copyrighted 2017 George Waldon
 */

package org.biobeans.modules.biocore.sequences;

import java.io.Serializable;


/** Sequence identifier; used to identify and compare sequences. This one is based
 *  on a single name.
 */
public class Identifier implements Comparable, Cloneable, Serializable {

    static final long serialVersionUID = 1L;

    /** Sequence name. */
    private String seqName;

    /** New identifier.
    * @param name name of the sequence
    */
    protected Identifier(String name) {
        String trim = name.trim();
        //trim = trim.intern();
        this.seqName = trim;
    }
    
    /** Create an identifier with a name and a resolver.
    *
    * @param seqName the sequence name to create the identifier
    * @return the identifier
    */
    public static Identifier create (String seqName) {
        return new Identifier(seqName);
    }

    /** Get the sequence name.
    */
    public String getName () { 
        return seqName;
    }

    /** Set the sequence name.
    */
    public void setName (String name) { 
        seqName = name;
    }
    /** Returns the name for text representation.
    * 
    */
    @Override
    public String toString() {
        return getName();
    }

    /** Compare the specified object with this Identifier on the sequence name.
    * @param o an Identifier casted to an Object.
    * @return ordering on sequence name.
    */
    @Override
    public int compareTo(Object o) {
        Identifier id = (Identifier)o;
        return getName().compareTo(id.getName());
    }

    /** Compare the specified object with this Identifier for equality.
    */
    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof Identifier))
            return false;
        return getName().equals( ((Identifier) o).getName());
    }

    /** @return the hash code of sequence name String object.
    */
    @Override
    public int hashCode() {
        return getName().hashCode();
    }
    
    @Override
    public Object clone() {
        return this;
    }
}
