/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or bbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * bbbuild/licenses/cddl-gpl-2-cp.txt.  This particular file is subject to
 * the "Classpath" exception as provided in the GPL Version 2 section of the
 * License file that accompanied this code. If applicable, add the following
 * below the License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 * Portions Copyrighted 2017 George Waldon
 */

package org.biobeans.modules.biocore.sequences;

import org.openide.filesystems.MIMEResolver;
import org.openide.util.NbBundle;

/** MIME types associated with simple sequence formats. Note, this MIME types are
 * purely internal and only exist to conform with NetBeans design to recognize
 * file content. 
 *
 * @author George Waldon
 */
public class MIMEType {
    
    private MIMEType() {
    }
    
    //====================== SEQUENCE FILES ==================================
    
    /** MIME type for raw sequence files.
     */
    public static final String RAW = "text/biology-seq-raw";
    @NbBundle.Messages("MimeResolverRAW=Raw Sequence Files")
    @MIMEResolver.ExtensionRegistration(
        displayName="#MimeResolverRAW",
        mimeType=RAW,
        extension={"seq","SEQ"},
        position=480
    )
    
    /** MIME type for FASTA file: "text/biology-seq-fasta"
     */
    public static final String FASTA = "text/biology-seq-fasta";
    @NbBundle.Messages("MimeResolverFASTA=FASTA Sequence Files")
    @MIMEResolver.ExtensionRegistration(
        displayName="#MimeResolverFASTA",
        mimeType=FASTA,
        extension={"fasta","fa","fas","faa","fsa"},
        position=500
    )
    
    /** MIME type for UniProt file: "text/biology-seq-uniprot";
     */
    public static final String UNIPROT = "text/biology-seq-uniprot";
    @NbBundle.Messages("MimeResolverUniProt=UniProt Sequence Files")
    @MIMEResolver.ExtensionRegistration(
        displayName="#MimeResolverUniProt",
        mimeType=UNIPROT,
        extension={"pro"},
        position=620
    )
    
    
    //PENDING text/*+xml is the standard form for xml recognition
    /** MIME type for INSD file: "text/xml-biology-seq-insd";
     */
    public static final String INSD = "text/xml-biology-seq-insd";
    @NbBundle.Messages("MimeResolverINSD=INSD Sequence Files")
    @MIMEResolver.NamespaceRegistration(
        displayName = "#MimeResolverINSD",
        checkedExtension = {"xml"},
        mimeType = INSD,
        doctypePublicId = {"-//NCBI//INSD INSDSeq/EN"},
        position = 600
    )
    
    /** MIME type for GenPept file: "text/biology-seq-genpept";
     */
    public static final String GENPEPT = "text/biology-seq-genpept";
    @NbBundle.Messages("MimeResolverGenPept=GenPept Sequence Files")
    @MIMEResolver.ExtensionRegistration(
        displayName="#MimeResolverGenPept",
        mimeType=GENPEPT,
        extension={"gp","genpept","genpep"},
        position=640
    )
    
    /** MIME type for GenBank file: "text/biology-seq-genbank";
     */
    public static final String GENBANK = "text/biology-seq-genbank";
    @NbBundle.Messages("MimeResolverGenBank=GenBank Sequence Files")
    @MIMEResolver.ExtensionRegistration(
        displayName="#MimeResolverGenBank",
        mimeType=GENBANK,
        extension={"gb","genbank"},
        position=660
    )
    
    /** MIME type for EMBL file: "text/biology-seq-embl";
     */
    public static final String EMBL = "text/biology-seq-embl";
    @NbBundle.Messages("MimeResolverEMBL=EMBL Sequence Files")
    @MIMEResolver.ExtensionRegistration(
        displayName="#MimeResolverEMBL",
        mimeType=EMBL,
        extension={"embl"},
        position=680
    )
    
    /** MIME type for Textco GCK binary file.
     */
    public static final String GCK = "application/biology-seq-gck";
    @NbBundle.Messages("MimeResolverGCK=Textco GCK Sequence Files")
    @MIMEResolver.ExtensionRegistration(
        displayName="#MimeResolverGCK",
        mimeType=GCK,
        extension={"gck","gcc"},
        position=490
    )
        
    public void types(){
    };

}
