/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or bbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * bbbuild/licenses/cddl-gpl-2-cp.txt.  This particular file is subject to
 * the "Classpath" exception as provided in the GPL Version 2 section of the
 * License file that accompanied this code. If applicable, add the following
 * below the License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 * Portions Copyrighted 2017 George Waldon
 */
package org.biobeans.modules.biocore.sequences;

import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;
import org.biojavax.Note;
import org.biojavax.SimpleNote;


public class NoteUtil {
    
    private NoteUtil() {};
    
    /** Test whether 2 notes are equal and have the same value.
     * 
     */
    public static boolean areNotesIdentical(Note note1, Note note2) {
        if(!note1.equals(note2))
            return false;
        return note1.getValue().equals(note2.getValue());
    }
    
    /** Test whether 2 sets of notes contain identical notes.
     * 
     */
    public static boolean areNoteSetsIdentical(TreeSet<Note> notes1, TreeSet<Note> notes2) {
        if(!notes1.equals(notes2))
            return false;
        Iterator<Note> it1 = notes1.iterator();
        Iterator<Note> it2 = notes2.iterator();
        while(it1.hasNext()) {
            Note n1 = it1.next();
            Note n2 = it2.next();
            if(!n1.getValue().equals(n2.getValue()))
                return false;
        }
        return true;
    }
    
    /** Verify that a set of notes is properly ordered. 
     */
    public static boolean isNoteSetOrdered(Set<Note> s) {
        if(s.size()<=1)
            return true;
        Iterator<Note> it = s.iterator();
        Comparable c1 = (Comparable) it.next();
        while(it.hasNext()) {
            Comparable c2 = (Comparable) it.next();
            if(c1.compareTo(c2)>0)
                return false;
            c1 = c2;
        }
        return true;
    }
    
    /** Clone a note.
     */
    public static Note cloneNote(Note note) {
        return new SimpleNote(
                    note.getTerm(),
                    note.getValue(),
                    note.getRank());
    }
    

    
}
