/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or bbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * bbbuild/licenses/cddl-gpl-2-cp.txt.  This particular file is subject to
 * the "Classpath" exception as provided in the GPL Version 2 section of the
 * License file that accompanied this code. If applicable, add the following
 * below the License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 * Portions Copyrighted 2017 George Waldon
 */

package org.biobeans.modules.biocore.sequences;

import java.io.IOException;
import org.biojava.bio.BioException;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.SequenceIterator;
import org.biojava.bio.seq.io.SeqIOConstants;

/** Cookie associated with sequence files. Sequences are parsed automatically
 * in the background unless the file is larger than a limit set by the user, in
 * which case the sequence iterator is the preferred method to access them.
 *
 * @author  George Waldon
 */
public interface SequenceCookie extends SeqProvider {
  
    /** Getter for sequence format.
     *
     * @return a {@SeqIOConstants} constant.
     */
    public int getSequenceFormat();
    
    /** Check the ability to change the sequence format
     */
    public boolean canSetFormat();
    
    /** Setter for the sequence format.
     *
     * @param format the new sequence format, a {@link SeqIOConstants} constant.
     * @param noEventFiring, if true, do not fire a property change.
     */
    public void setSequenceFormat( int format, boolean noEventFiring );
    
    /** Getter for sequences as a sequence array
     */
    @Override
    public Sequence[] getSequences();
    
    /** Getter of an iterator over sequences.
     */   
    public SequenceIterator getSequenceIterator() throws IOException, BioException;
    
}
