/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or bbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * bbbuild/licenses/cddl-gpl-2-cp.txt.  This particular file is subject to
 * the "Classpath" exception as provided in the GPL Version 2 section of the
 * License file that accompanied this code. If applicable, add the following
 * below the License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 * Portions Copyrighted 2017 George Waldon
 */
package org.biobeans.modules.biocore.sequences;

import org.biojava.bio.BioException;
import org.biojava.bio.seq.io.ParseException;
import org.biojavax.RichObjectFactory;
import org.biojavax.bio.seq.RichLocation;
import org.biojavax.bio.seq.io.GenbankLocationParser;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author gwaldon
 */
public class LocationUtilTest {
    
    public LocationUtilTest() {
    }
    
    @BeforeClass
    public static void setUpClass() {
    }
    
    @AfterClass
    public static void tearDownClass() {
    }
    
    @Before
    public void setUp() {
    }
    
    @After
    public void tearDown() {
    }
    
    private RichLocation parseLocation(String val) {
        RichLocation rloc = null;
        try {
            rloc = GenbankLocationParser.parseLocation(RichObjectFactory.getDefaultNamespace(), "", val);
        } catch (ParseException ex) {
            System.out.println("parsong error; fix the location: " + rloc);
        }
        return rloc;
    }
    
    @Test
    public void testgetLocationStart() {
        String val = "join(12..78,134..202)";
        String val1= "join(134..202,12..78)";
        String val2 = "complement(join(12..78,134..202))";
        String val3 = "complement(join(134..202,12..78))";
        
        int start = 12;
        int start1 = 134;
        int start2 = 202;
        int start3 = 78;
       
        RichLocation rloc = parseLocation(val);
        int locstart = LocationUtil.getLocationStart(rloc);
        assertEquals(start, locstart);

        RichLocation rloc1 = parseLocation(val1);
        int locstart1 = LocationUtil.getLocationStart(rloc1);
        assertEquals(start1, locstart1);

        RichLocation rloc2 = parseLocation(val2);
        int locstart2 = LocationUtil.getLocationStart(rloc2);
        assertEquals(start2, locstart2);

        RichLocation rloc3 = parseLocation(val3);
        int locstart3 = LocationUtil.getLocationStart(rloc3);
        assertEquals(start3, locstart3);

        System.out.println("getLocationStart verified");
    }
    
    @Test
    public void testgetLocationEnd() {
        String val = "join(12..78,134..202)";
        String val1= "join(134..202,12..78)";
        String val2 = "complement(join(12..78,134..202))";
        String val3 = "complement(join(134..202,12..78))";
        
        int end = 202;
        int end1 = 78;
        int end2 = 12;
        int end3 = 134;

        RichLocation rloc = parseLocation(val);
        int locstart = LocationUtil.getLocationEnd(rloc);
        assertEquals(end, locstart);

        RichLocation rloc1 = parseLocation(val1);
        int locstart1 = LocationUtil.getLocationEnd(rloc1);
        assertEquals(end1, locstart1);

        RichLocation rloc2 = parseLocation(val2);
        int locstart2 = LocationUtil.getLocationEnd(rloc2);
        assertEquals(end2, locstart2);

        RichLocation rloc3 = parseLocation(val3);
        int locstart3 = LocationUtil.getLocationEnd(rloc3);
        assertEquals(end3, locstart3);

        System.out.println("getLocationEnd verified");
    }

    private boolean verifyLocationOrder(String loc, boolean cancross) {
        RichLocation rloc = null;
        try {
            rloc = GenbankLocationParser.parseLocation(RichObjectFactory.getDefaultNamespace(), "", loc);
        } catch (ParseException ex) {
            System.out.println("parsong error; fix the location: "+loc);
        }
            return LocationUtil.verifyLocationOrder(rloc,cancross);
    }
    
    /**
     * Test of verifyLocationOrder method, of class LocationUtil.
     */
    @Test
    public void testVerifyLocationOrder() {
                
        String val = "join(12..78,134..202)";
        String val1= "join(95..102,134..202,12..78,80..92)";
        String val2= "join(134..202,12..78,1..3)";
        String val3 = "complement(join(12..78,134..202))";
        String val4= "complement(join(95..102,134..202,12..78,80..92))";
        String val5= "complement(join(134..202,12..78,1..3))";
        String val6 = "join(12..78,77..202)";
        String val7 = "complement(join(12..78,77..202))";
        String val8 = "join(77..202,12..78)";
        String val9 = "20..12";
        String val10 = "complement(20..12)";

        //Positive strand in order
        assertTrue(verifyLocationOrder(val, true));
        //Positive strand in order crossing the origin
        assertTrue(verifyLocationOrder(val1, true));
        //multiple crossing
        assertFalse(verifyLocationOrder(val2, true));
        //Negative strand in order
        assertTrue(verifyLocationOrder(val3, true));
        //Positive strand in order crossing the origin
        assertTrue(verifyLocationOrder(val4, true));
        //multiple crossing
        assertFalse(verifyLocationOrder(val5, true));

        //overlap - indeed catched by the parser.
        //assertFalse(verifyLocationOrder(val6,true));
        //assertFalse(verifyLocationOrder(val7,true));
        //assertFalse(verifyLocationOrder(val8,true));
        //inverted location
        assertFalse(verifyLocationOrder(val9, true));
        assertFalse(verifyLocationOrder(val10, true));

        //Positive strand in order
        assertTrue(verifyLocationOrder(val, false));
        //Positive strand in order crossing the origin
        assertFalse(verifyLocationOrder(val1, false));
        //multiple crossing
        assertFalse(verifyLocationOrder(val2, false));
        //Negative strand in order
        assertTrue(verifyLocationOrder(val3, false));
        //Positive strand in order crossing the origin
        assertFalse(verifyLocationOrder(val4, false));
        //multiple crossing
        assertFalse(verifyLocationOrder(val5, false));
        //inverted location
        assertFalse(verifyLocationOrder(val9, false));
        assertFalse(verifyLocationOrder(val10, false));

        System.out.println("verifyLocationOrder verified");
    }

    private boolean checkorder(String val1, String val2) {
        RichLocation[] rlocs = new RichLocation[2];
        try {
            RichLocation loc1 = GenbankLocationParser.parseLocation(RichObjectFactory.getDefaultNamespace(), "", val1);
            loc1 = LocationUtil.orderLocation(loc1);
            rlocs[0] = loc1;
            assertTrue(LocationUtil.verifyLocationOrder(loc1,true));
            RichLocation loc2 = GenbankLocationParser.parseLocation(RichObjectFactory.getDefaultNamespace(), "", val2);
            rlocs[1] = loc2;
        } catch (ParseException ex) {
            System.out.println("parsong error; fix the location");
        }
        return rlocs[0].equals(rlocs[1]);
    }
    
    /**
     * Test of orderLocation method, of class LocationUtil.
     */
    @Test
    public void testOrderLocation() {
        String val = "join(12..78,134..202)";
        String val1 = "join(134..202,12..78)";
        String val2 = "join(12..78,79..202)";
        String valout2 = "12..202";
        String val3 = "join(12..78,79..202,203..203,204..402)";
        String valout3 = "12..402";
        
        String val4 = "complement(join(12..78,134..202))";
        String val5 = "complement(join(134..202,12..78))";
        String val6 = "complement(join(12..78,79..202))";
        String valout6 = "complement(12..202)";
        String val7 = "complement(join(12..78,79..202,203..203,204..402))";
        String valout7 = "complement(12..402)";

        assertTrue(checkorder(val,val));
        assertTrue(checkorder(val1, val));
        assertTrue(checkorder(val2, valout2));
        assertTrue(checkorder(val3, valout3));
        
        assertTrue(checkorder(val4, val4));
        assertTrue(checkorder(val5, val4));
        assertTrue(checkorder(val6, valout6));
        assertTrue(checkorder(val7, valout7));

        System.out.println("orderLocation verified");
    }

    /**
     * Test of excludeLocation method, of class LocationUtil.
     */
    @Test
    public void testExcludeLocation() {
        int cirlen = 798;
        
        String val = "join(12..78,134..202)";
        String exclude = "1..80";
        String result = "134..202";
        
        RichLocation rloc = parseLocation(val);
        RichLocation rlocexclude = parseLocation(exclude);
        RichLocation resloc = LocationUtil.excludeLocation(rloc, rlocexclude);
        assertTrue(LocationUtil.verifyLocationOrder(resloc,true));
        RichLocation rlocresult = parseLocation(result);
        assertEquals(resloc, rlocresult);
        
        rloc.setCircularLength(cirlen);
        rlocexclude.setCircularLength(cirlen);
        resloc = LocationUtil.excludeLocation(rloc, rlocexclude);
        assertTrue(LocationUtil.verifyLocationOrder(resloc,true));
        rlocresult = parseLocation(result);
        assertEquals(resloc, rlocresult);
        
        String val1 = "join(12..78,134..202)";
        String exclude1 = "1..77";
        String result1 = "join(78,134..202)";
        
        RichLocation rloc1 = parseLocation(val1);
        RichLocation rlocexclude1 = parseLocation(exclude1);
        RichLocation resloc1 = LocationUtil.excludeLocation(rloc1, rlocexclude1);
        assertTrue(LocationUtil.verifyLocationOrder(resloc1,true));
        RichLocation rlocresult1 = parseLocation(result1);
        assertEquals(resloc1, rlocresult1);
        
        String val2 = "complement(join(12..78,134..202))";
        String exclude2 = "complement(1..80)";
        String result2 = "complement(134..202)";
        
        RichLocation rloc2 = parseLocation(val2);
        RichLocation rlocexclude2 = parseLocation(exclude2);
        RichLocation resloc2 = LocationUtil.excludeLocation(rloc2, rlocexclude2);
        assertTrue(LocationUtil.verifyLocationOrder(resloc2,true));
        RichLocation rlocresult2 = parseLocation(result2);
        assertEquals(resloc2, rlocresult2);
        
        String val3 = "complement(join(12..78,134..202))";
        String exclude3 = "complement(1..77)";
        String result3 = "complement(join(78,134..202))";
        
        RichLocation rloc3 = parseLocation(val3);
        RichLocation rlocexclude3 = parseLocation(exclude3);
        RichLocation resloc3 = LocationUtil.excludeLocation(rloc2, rlocexclude3);
        assertTrue(LocationUtil.verifyLocationOrder(resloc3,true));
        RichLocation rlocresult3 = parseLocation(result3);
        assertEquals(resloc3, rlocresult3);
        
        String val4 = "join(12..78,134..202,500..798)";
        String exclude4 = "77..134";
        String result4 = "join(12..76,135..202,500..798)";
        
        RichLocation rloc4 = parseLocation(val4);
        RichLocation rlocexclude4 = parseLocation(exclude4);
        RichLocation resloc4 = LocationUtil.excludeLocation(rloc4, rlocexclude4);
        assertTrue(LocationUtil.verifyLocationOrder(resloc4,true));
        RichLocation rlocresult4 = parseLocation(result4);
        assertEquals(resloc4, rlocresult4);
        
        String val5 = "join(500..798,12..78,134..202)";
        String exclude5 = "join(701..798";
        String result5 = "join(500..700,12..78,134..202)";
        
        RichLocation rloc5 = parseLocation(val5);
        RichLocation rlocexclude5 = parseLocation(exclude5);
        RichLocation resloc5 = LocationUtil.excludeLocation(rloc5, rlocexclude5);
        assertTrue(LocationUtil.verifyLocationOrder(resloc5,true));
        RichLocation rlocresult5 = parseLocation(result5);
        assertEquals(resloc5, rlocresult5);
        
        System.out.println("excludeLocation verified");
    }

    /**
     * Test of joinNsortLocation method, of class LocationUtil.
     */
    @Test
    public void testJoinLocation() {
        
        String val = "join(12..78,134..202)";
        String toadd = "100..105";
        String result = "join(12..78,100..105,134..202)";
        
        RichLocation rloc = parseLocation(val);
        RichLocation rloctoadd = parseLocation(toadd);
        RichLocation joinloc = null;
        try {
            joinloc = LocationUtil.joinLocation(rloc, rloctoadd);
        } catch (BioException ex) {
            fail("A BioException has been thown");
        }
        assertTrue(LocationUtil.verifyLocationOrder(joinloc,true));
        RichLocation rlocresult = parseLocation(result);
        assertEquals(joinloc, rlocresult);
     
        String val1 = "complement(join(12..78,134..202))";
        String toadd1 = "complement(100..105)";
        String result1 = "complement(join(12..78,100..105,134..202)";
        
        RichLocation rloc1 = parseLocation(val1);
        RichLocation rloctoadd1 = parseLocation(toadd1);
        RichLocation joinloc1 = null;
        try {
            joinloc1 = LocationUtil.joinLocation(rloc1, rloctoadd1);
        } catch (BioException ex) {
            fail("A BioException has been thown");
        }
        assertTrue(LocationUtil.verifyLocationOrder(joinloc1,true));
        RichLocation rlocresult1 = parseLocation(result1);
        assertEquals(joinloc1, rlocresult1);
        
        RichLocation unionloc1 = RichLocation.Tools.enrich(rloc.union(rloctoadd1));
        //BioJava bug
        //Does not take care of rank properly
        assertFalse(LocationUtil.verifyLocationOrder(unionloc1,true));
        
        //Circularity
        int cirlen = 798;
        rloc1.setCircularLength(cirlen);
        rloctoadd1.setCircularLength(cirlen);
        rlocresult1.setCircularLength(cirlen);
        try {
            joinloc1 = LocationUtil.joinLocation(rloc1, rloctoadd1);
        } catch (BioException ex) {
            fail("A BioException has been thown");
        }
        assertEquals(joinloc1, rlocresult1);
        
        //Difficult case... Where is the beginning of the union location?
        String val2 = "join(134..202,12..78)";
        String toadd2 = "100..105";
        //Simplest solution, simply order from origin.
        String result2 = "join(12..78,100..105,134..202)";
        
        RichLocation rloc2 = parseLocation(val2);
        RichLocation rloctoadd2 = parseLocation(toadd2);
        RichLocation joinloc2 = null;
        try {
            joinloc2 = LocationUtil.joinLocation(rloc2, rloctoadd2);
        } catch (BioException ex) {
            fail("A BioException has been thrown");
        }
        assertTrue(LocationUtil.verifyLocationOrder(joinloc2,true));
        RichLocation rlocresult2 = parseLocation(result2);
        assertEquals(joinloc2, rlocresult2);
        

        System.out.println("joinLocation verified");
    }

    /**
     * Test of getReverseComplementLocation method, of class LocationUtil.
     */
    @Test
    public void testGetReverseComplementLocation() {
    }

    /**
     * Test of getComplementLocation method, of class LocationUtil.
     */
    @Test
    public void testGetComplementLocation() {
    }

    /**
     * Test of translateLocation method, of class LocationUtil.
     */
    @Test
    public void testTranslateLocation() {
    }

    /**
     * Test of cloneLocation method, of class LocationUtil.
     */
    @Test
    public void testCloneLocation() {
        String val = "join(12..78)";
        String val1 = "join(12..78,134..202)";
        
        //Simple
        RichLocation rloc = null;
        try {
            rloc = GenbankLocationParser.parseLocation(RichObjectFactory.getDefaultNamespace(), "", val);
        } catch (ParseException ex) {
            System.out.println("parsong error; fix the location: "+val);
        }
        RichLocation copy = LocationUtil.cloneLocation(rloc);
        assertEquals(copy,rloc);
        
        //Compound
        RichLocation rloc1 = null;
        try {
            rloc1 = GenbankLocationParser.parseLocation(RichObjectFactory.getDefaultNamespace(), "", val1);
        } catch (ParseException ex) {
            System.out.println("parsong error; fix the location: "+val1);
        }
        RichLocation copy1 = LocationUtil.cloneLocation(rloc1);
        assertEquals(copy1,rloc1);
        
        System.out.println("cloneLocation verified");
    }

    /**
     * Test of crossOrigin method, of class LocationUtil.
     */
    @Test
    public void testCrossOrigin() {
    }

    /**
     * Test of isLocationContigousAcrossOrigin method, of class LocationUtil.
     */
    @Test
    public void testIsLocationContigousAcrossOrigin() {

    }
    
}
