/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or bbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * bbbuild/licenses/cddl-gpl-2-cp.txt.  This particular file is subject to
 * the "Classpath" exception as provided in the GPL Version 2 section of the
 * License file that accompanied this code. If applicable, add the following
 * below the License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 * Portions Copyrighted 2013 George Waldon
 */
package org.biobeans.modules.biotoolbar;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.prefs.PreferenceChangeEvent;
import java.util.prefs.PreferenceChangeListener;
import javax.swing.AbstractAction;
import javax.swing.DefaultComboBoxModel;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.text.JTextComponent;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionReferences;
import org.openide.awt.ActionRegistration;
import org.openide.awt.HtmlBrowser;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;
import org.openide.util.NbBundle.Messages;
import org.openide.util.actions.Presenter;

/** An action to query a bioinformation center.
 * 
 * @author George Waldon
 */
@ActionID(category = "Biology", id = "org.biobeans.modules.biotoolbar.BioToolbarAction")
@ActionRegistration(displayName = "#CTL_biotoolbar")
@ActionReferences({
    @ActionReference(path = "Toolbars/BioToolbar", position = 100)
})
@Messages("CTL_biotoolbar=BioToolbar")
public class BioToolbarAction extends AbstractAction implements Presenter.Toolbar {

    private static final Logger LOGGER = Logger.getLogger(BioToolbarAction.class.getName());
    
    // List of center names
    static List<String> centers;
    
    // The list of searches for each center will be encoded in a single String
    // in the form "a|b|c" where a is the action, b the query and c the option.
    static Map<String,List<String>> searches;
    
    // List of display names associated with the list of centers
    static Map<String,String> centernames;
    
    /** Maximum number of row to display in the search box. */
    public final static int MAXROW = 20;
    
    private int preferredBoxWidth = 80;
    private int preferredInputWidth = 360;
    private ComboBoxMenu centerChooser;
    private JComboBox searchChooser;
    private JComboBox inputField;
    private PreferenceChangeListener settingsListener;
        
    public BioToolbarAction() {
        // See # 200522 
        super(NbBundle.getMessage(BioToolbarAction.class, "CTL_biotoolbar"));
    }
    
    @Override
    public void actionPerformed(ActionEvent e) {
    }

    @Override
    public Component getToolbarPresenter() {
        JPanel pan = new JPanel();
        GridBagConstraints c = new GridBagConstraints();
        c.insets = new Insets(2, 2, 2, 2);
        GridBagLayout layout = new GridBagLayout();
        layout.setConstraints(pan, c);
        pan.setLayout(layout);

        String labelText = "<html><font color=\"gray\"><b>&nbsp;" + //NOI18N
                NbBundle.getMessage(BioToolbarAction.class, "LBL_search")
                + "&nbsp;</b></font></html>"; //NOI18N
        pan.add(new JLabel(labelText));

        centerChooser = new ComboBoxMenu();
        centerChooser.addActionListener(new java.awt.event.ActionListener() {

            @Override
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                BioToolbarAction.this.actionCenterChooserPerformed(evt);
            }
        });
        pan.add(centerChooser, c);

        searchChooser = new JComboBox();
        searchChooser.setMaximumRowCount(MAXROW);
        pan.add(searchChooser, c);

        labelText = "<html><font color=\"gray\"><b>&nbsp;&nbsp;" + //NOI18N
                NbBundle.getMessage(BioToolbarAction.class, "LBL_for")
                + "&nbsp;&nbsp;</b></font></html>"; //NOI18N
        pan.add(new JLabel(labelText));
        
        inputField = new JComboBox();
        inputField.setEditable(true);
        inputField.setMaximumRowCount(MAXROW);
        JTextComponent tc = (JTextComponent) inputField.getEditor().getEditorComponent();
        tc.addKeyListener(new KeyListener() {

            @Override
            public void keyTyped(KeyEvent e) {
            }

            @Override
            public void keyPressed(KeyEvent e) {
            }

            @Override
            public void keyReleased(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                    BioToolbarAction.this.inputPerformed();
                }
            }
        });
        Dimension dim = inputField.getPreferredSize();
        if (dim.width < preferredInputWidth) {
            dim.width = preferredInputWidth;
            inputField.setPreferredSize(dim);
        }
        pan.add(inputField, c);
        
        Icon iq = ImageUtilities.loadImageIcon("org/biobeans/modules/biotoolbar/resources/search.png", false); //NOI18N  
        JButton queryButton = new JButton(iq);
        queryButton.setToolTipText(NbBundle.getMessage(BioToolbarAction.class, "CTL_query"));
        dim = queryButton.getPreferredSize();
        dim.width = dim.height;
        queryButton.setPreferredSize(dim);
        queryButton.addActionListener(new ActionListener(){

            @Override
            public void actionPerformed(ActionEvent e) {
                String q = (String) inputField.getSelectedItem();
                queryDatabase(q);
            }
        });
        pan.add(queryButton);
                
        SwingUtilities.invokeLater(new Runnable() {

            @Override
            public void run() {
                initToolbar();
            }
           
        });
        
        settingsListener = new PreferenceChangeListener() {

            @Override
            public void preferenceChange(PreferenceChangeEvent evt) {
                if (QuerySettings.PROP_BIOINFORMATIONCENTER.equals(evt.getKey())) {
                    populateCenters();
                } else if (QuerySettings.PROP_SEARCH.equals(evt.getKey())) {
                    populateCenters();
                }
            }
        };
        QuerySettings.getDefault().addPreferenceChangeListener(settingsListener);
        return pan;
    }

    private void actionCenterChooserPerformed(ActionEvent evt) {
        ComboMenuBar bar = (ComboMenuBar)evt.getSource();
        String center = bar.menu.getName();
        populateSearches(center);
    }
    
    public void populateSearches(String center) {
        ((DefaultComboBoxModel) searchChooser.getModel()).removeAllElements();
        int ind = centers.indexOf(center);
        if(ind==-1)
            return;
        List<String> snames = searches.get(center);
        if(snames.isEmpty())
            return;
        for(String s:snames) {
            String sn = s.substring(0,s.indexOf('|')); 
            searchChooser.addItem(sn);
        }
        
        String defaultCenter = QuerySettings.getDefault().getBioinformationCenter();
        int sind = 0;
        if(defaultCenter.equals(center)) {
            String defaultSearch = QuerySettings.getDefault().getSearch();
            for(String sn:snames) {
                String sd = sn.substring(0,sn.indexOf('|'));
                if(sd.equals(defaultSearch)) {
                    sind = snames.indexOf(sn);
                }
            }
        }
        searchChooser.setSelectedIndex(sind);
    }
    
    FileObject getToolbarFolder() {
        FileObject toolbarFolder;
        try {
            toolbarFolder = FileUtil.getConfigFile("BioToolbar"); // NOI18N
            if (toolbarFolder == null)
                toolbarFolder = FileUtil.getConfigRoot().createFolder("BioToolbar"); // NOI18N
        }
        catch (java.io.IOException ex) {
            IllegalStateException e = new IllegalStateException("BioTolbar folder not found and cannot be created.",ex); //NOI18N
            LOGGER.log(Level.WARNING,"",e);
            throw e;
        }
        return toolbarFolder;
    }
    
    private void addCenter(DataFolder df, JMenu menu, boolean create) {
        FileObject fo = df.getPrimaryFile();
        String cname = fo.getName();
        String dname = (String) fo.getAttribute("displayName"); // NOI18N
        String base = (String) fo.getAttribute("iconBase"); // NOI18N
        if(create)
            centers.add(cname);
        JMenuItem item = new JMenuItem(dname);
        item.setName(cname);
        if (base != null) {
            item.setIcon(ImageUtilities.loadImageIcon(base, true));
        }
        menu.add(item);
        
        if(create) {
            List<String> search = new ArrayList<String>();
            DataObject[] dfs = df.getChildren();
            for (int i = 0; i < dfs.length; i++) {
                FileObject fo2 = dfs[i].getPrimaryFile();
                Boolean s = (Boolean) fo2.getAttribute("search"); // NOI18N
                if(s!=null && s==true) {
                    String name2 = (String) fo2.getAttribute("displayName"); // NOI18N
                    String query = (String) fo2.getAttribute("query"); // NOI18N
                    String option = (String) fo2.getAttribute("option"); // NOI18N
                    assert assertValue(name2);
                    assert assertValue(query);
                    assert assertValue(option);
                    query = name2 + "|" + query + "|" + option;
                    search.add(query);
                }
            }
            searches.put(cname, search);
            centernames.put(cname, dname);
        }
    }
    
    private boolean assertValue(String v) {
        boolean isNull = v==null;
        if(isNull)
            assert false : "BioToolbar: " + v + " null, check attribute displayName"; //NOI18N //BREAKPOINT
        return !isNull;
    }
    
    private void initFolder(DataFolder df, JMenu menu, boolean create) {
        FileObject fo = df.getPrimaryFile();
        Boolean center = (Boolean) fo.getAttribute("center"); // NOI18N
        if (center!=null && center == true) {
            addCenter(df, menu,create);
        } else {
            String name = (String) fo.getAttribute("displayName"); // NOI18N
            JMenu menu2 = new JMenu(name);
            menu.add(menu2);
            DataObject[] dos = df.getChildren();
            for (int i = 0; i < dos.length; i++) {
                if (dos[i] instanceof DataFolder) {
                    FileObject fo2 = dos[i].getPrimaryFile();
                    Boolean center2 = (Boolean) fo2.getAttribute("center"); // NOI18N
                    if (center2!=null && center2 == true) {
                        addCenter((DataFolder) dos[i], menu2,create);
                    } else {
                        initFolder((DataFolder) dos[i], menu2,create);
                    }
                } else {
                    // Should not happend
                }
            }
        }
    }
    
    private void initToolbarFolder() {
        boolean create = false;
        if(centers==null) {
            centers = new ArrayList<String>();
            searches = new HashMap<String,List<String>>();
            centernames = new HashMap<String,String>();
            create = true;
        }
        FileObject folder = getToolbarFolder();
        DataFolder df = DataFolder.findFolder(folder);
        DataObject[] dfs = df.getChildren();
        for (int i=0; i<dfs.length; i++) {
            if (dfs[i] instanceof DataFolder)
                initFolder((DataFolder)dfs[i],centerChooser.getTopMenu(),create);
        }
    }
    
    private void initToolbar() {
        initToolbarFolder();
        
        centerChooser.setListener();
        // Adjust size of combo to size of menus
        Dimension dim = centerChooser.getPreferredSize();
        if (dim.width < preferredBoxWidth) {
            dim.width = preferredBoxWidth;
        }
        centerChooser.setPreferredSize(dim);
        
        // Load default center
        populateCenters();
    }
    
    private void populateCenters() {
        String defaultCenter = QuerySettings.getDefault().getBioinformationCenter();
        int ind = centers.indexOf(defaultCenter);
        if(ind==-1) {
            ind = 0;
            defaultCenter = centers.get(0);
        }
        centerChooser.setText(centernames.get(defaultCenter));
        centerChooser.getTopMenu().setName(defaultCenter);
        populateSearches(defaultCenter);
    }
    
    private void inputPerformed() {
        String ip = (String)inputField.getSelectedItem();
        if(ip==null || ip.trim().isEmpty())
            return;
        boolean alreadyThere = false;
        for (int i = 0; i < inputField.getItemCount(); i++) {
            if (inputField.getItemAt(i).toString().equals(ip)) {
                alreadyThere = true;
                break;
            }
        }
        if (!alreadyThere) {
            inputField.addItem(ip);
        }
        queryDatabase(ip);
    }
    
    private void queryDatabase(String input) {
        if(input==null || input.isEmpty())
            return;
        String center = centerChooser.getTopMenu().getName();
        if(center.isEmpty())
            return;
        List<String> snames = searches.get(center);
        int is = searchChooser.getSelectedIndex();
        if(is==-1)
            return;
        String q = snames.get(is);
        String[] es = q.split("\\|");
        
        String query="";
        if(es.length==3)
            query = MessageFormat.format(es[1],input,es[2]);
        else if(es.length==2)
            query = MessageFormat.format(es[1],input);
        performQuery(query);
        
    }
    
    private void performQuery(final String query) {
        Runnable r = new Runnable() {

            @Override
            public void run() {
                URL url = getURLFromString(query);
                HtmlBrowser.URLDisplayer.getDefault().showURL(url);
            }
        };
        SwingUtilities.invokeLater(r);
    }
    
    private URL getURLFromString(String urlString) {
        try {
            return new URL(urlString);
        } catch (MalformedURLException mue1) {
        }

        try {
            return new URL("http://" + urlString); //NOI18N
        } catch (MalformedURLException mue1) {
        }
        return null;
    }
}
