/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or bbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * bbbuild/licenses/cddl-gpl-2-cp.txt.  This particular file is subject to
 * the "Classpath" exception as provided in the GPL Version 2 section of the
 * License file that accompanied this code. If applicable, add the following
 * below the License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 * Portions Copyrighted 2013 George Waldon
 */

package org.biobeans.modules.biotoolbar;

import java.util.List;
import java.util.Map;
import javax.swing.DefaultComboBoxModel;

public class BioToolbarCustomizer extends javax.swing.JPanel {

    private boolean changed = false;
    private boolean isUpdating = false;
    
    public BioToolbarCustomizer() {
        initComponents();
        centerComboBox.setMaximumRowCount(BioToolbarAction.MAXROW);
        searchComboBox.setMaximumRowCount(BioToolbarAction.MAXROW);
    }
    
    private void populateSearches(String center) {
        ((DefaultComboBoxModel) searchComboBox.getModel()).removeAllElements();
        if(center==null)
            return;
        int ind  = 0;
        List<String> centers = BioToolbarAction.centers;
        if(centers!=null) {
            ind = centers.indexOf(center);
        }
        if(ind==-1)
            return;
        Map<String,List<String>> searches = BioToolbarAction.searches;
        List<String> snames = searches.get(center);
        for(String s:snames) {
            String sn = s.substring(0,s.indexOf('|')); 
            searchComboBox.addItem(sn);
        }
        
        String defaultCenter = QuerySettings.getDefault().getBioinformationCenter();
        int sind = 0;
        if(defaultCenter.equals(center)) {
            String defaultSearch = QuerySettings.getDefault().getSearch();
            for(String sn:snames) {
                String sd = sn.substring(0,sn.indexOf('|'));
                if(sd.equals(defaultSearch)) {
                    sind = snames.indexOf(sn);
                }
            }
        }
        searchComboBox.setSelectedIndex(sind);
    }
    
    boolean isChanged() {
        return changed;
    }

    boolean isDataValid() {
        return true;
    }

    void update() {
        isUpdating = true;
        ((DefaultComboBoxModel) centerComboBox.getModel()).removeAllElements();
        List<String> centers = BioToolbarAction.centers;
        Map<String,String> centernames = BioToolbarAction.centernames;
        if(centers!=null && centernames!=null) {
            for(String cn:centers) {
                String dn = centernames.get(cn);
                centerComboBox.addItem(dn);
            }
            String defaultCenter = QuerySettings.getDefault().getBioinformationCenter();
            int ind = centers.indexOf(defaultCenter);
            if(ind==-1) ind=0;
            centerComboBox.setSelectedIndex(ind);
            String c = centers.get(ind);
            populateSearches(c);
        }
        changed = false;
        isUpdating = false;
    }
    
    void applyChanges() {
        if(changed==true) {
            int sind = centerComboBox.getSelectedIndex();
            if(sind==-1)
                return;
            List<String> centers = BioToolbarAction.centers;
            if(centers==null)
                return;
            String center = centers.get(sind);
            QuerySettings.getDefault().setBioinformationcenter(center);
            sind = searchComboBox.getSelectedIndex();
            if(sind==-1)
                return;
            Map<String,List<String>> searches = BioToolbarAction.searches;
            if(searches==null)
                return;
            List<String> snames = searches.get(center);
            String sn = snames.get(sind);
            String sn2 = sn.substring(0,sn.indexOf('|'));
            QuerySettings.getDefault().setSearch(sn2);
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        centerComboBox = new javax.swing.JComboBox();
        searchComboBox = new javax.swing.JComboBox();
        centerLabel = new javax.swing.JLabel();
        searchLabel = new javax.swing.JLabel();

        centerComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        centerComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                centerComboBoxActionPerformed(evt);
            }
        });

        searchComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        searchComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                searchComboBoxActionPerformed(evt);
            }
        });

        centerLabel.setText(org.openide.util.NbBundle.getMessage(BioToolbarCustomizer.class, "BioToolbarCustomizer.centerLabel.text")); // NOI18N

        searchLabel.setText(org.openide.util.NbBundle.getMessage(BioToolbarCustomizer.class, "BioToolbarCustomizer.searchLabel.text")); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(searchLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(searchComboBox, 0, 167, Short.MAX_VALUE)
                    .addComponent(centerComboBox, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(centerLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 249, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(129, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(centerLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(centerComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(searchLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(searchComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(191, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void centerComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_centerComboBoxActionPerformed
        if(isUpdating)
            return;
        String command = evt.getActionCommand();
        if(command!=null && command.equals("comboBoxChanged")) {
            int sind = centerComboBox.getSelectedIndex();
            if(sind!=-1) {
                List<String> centers = BioToolbarAction.centers;
                if(centers!=null) {
                    String center = centers.get(sind);
                    populateSearches(center);
                } else
                    populateSearches(null);
                changed = true;
            }
        }
    }//GEN-LAST:event_centerComboBoxActionPerformed

    private void searchComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_searchComboBoxActionPerformed
        if(isUpdating)
            return;
        String command = evt.getActionCommand();
        if(command!=null && command.equals("comboBoxChanged")) {
            int sind = searchComboBox.getSelectedIndex();
            if(sind!=-1)
                changed = true;
        }
    }//GEN-LAST:event_searchComboBoxActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JComboBox centerComboBox;
    private javax.swing.JLabel centerLabel;
    private javax.swing.JComboBox searchComboBox;
    private javax.swing.JLabel searchLabel;
    // End of variables declaration//GEN-END:variables
}
