package org.biobeans.modules.biotoolbar;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.logging.Logger;
import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.SwingConstants;
import javax.swing.plaf.basic.BasicArrowButton;

public class ComboBoxMenu extends JComponent {

    private static final Logger LOGGER = Logger.getLogger(ComboBoxMenu.class.getName());
    
    ComboMenuBar bar;
    
    public ComboBoxMenu() {
        JMenu menu = ComboMenuBar.createMenu(QuerySettings.getDefault().getBioinformationCenter());
        setLayout(new FlowLayout());
        bar = new ComboMenuBar(menu);
        bar.setBorder(BorderFactory.createEtchedBorder());
        add(bar);
    }
    
    public JMenu getTopMenu() {
        return bar.menu;
    }
    
    public void setListener() {
        bar.setListener();
    }
    
    public void setText(String text) {
        bar.menu.setText(text);
    }
    
    public String getText() {
        return bar.menu.getText();
    }
    
    public void addActionListener(ActionListener l) {
        bar.addActionListener(l);
    }

    public void removeActionListener(ActionListener l) {
	bar.removeActionListener(l);
    }
    
    /** Set preferred size according to menu preferred
     *  size; menu must be set already to be effective.
     */
    @Override
    public void setPreferredSize(Dimension dim) {
        Dimension size = bar.getPreferredSize();
        size.width = Math.max(size.width, dim.width);
        bar.setPreferredSize(size);
        
        Dimension d = getPreferredSize();
        d.width = Math.max(size.width, d.width);
        super.setPreferredSize(d);
    }
}

class ComboMenuBar extends JMenuBar {

    JMenu menu;
    Dimension preferredSize;

    public ComboMenuBar(JMenu menu) {
        this.menu = menu;
        add(menu);
    }
    
    void setListener() {
        MenuItemListener listener = new MenuItemListener();
        setListener(menu, listener);
    }
    
    public void addActionListener(ActionListener l) {
        listenerList.add(ActionListener.class,l);
    }

    public void removeActionListener(ActionListener l) {
	listenerList.remove(ActionListener.class, l);
    }
    
    protected void fireActionEvent() {
        Object[] listeners = listenerList.getListenerList();
        for (int i = listeners.length - 2; i >= 0; i -= 2) {
            if (listeners[i] == ActionListener.class) {
                // Lazily create the event:
                ActionEvent e = new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "", 0, 0);
                ((ActionListener) listeners[i + 1]).actionPerformed(e);
            }
        }
    }
    
    class MenuItemListener implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            JMenuItem item = (JMenuItem) e.getSource();
            menu.setText(item.getText());
            menu.setName(item.getName());
            menu.requestFocus();
            fireActionEvent();
        }
    }

    private void setListener(JMenuItem item, ActionListener listener) {
        if (item instanceof JMenu) {
            JMenu imenu = (JMenu) item;
            int n = imenu.getItemCount();
            for (int i = 0; i < n; i++) {
                setListener(imenu.getItem(i), listener);
            }
        } else if (item != null) { // null means separator
            item.addActionListener(listener);
        }
    }

    @Override
    public Dimension getPreferredSize() {
        if (preferredSize == null) {
            Dimension sd = super.getPreferredSize();
            Dimension menuD = getItemSize(menu);
            Insets margin = menu.getMargin();
            Dimension retD = new Dimension(menuD.width, margin.top
                    + margin.bottom + menuD.height);
            menu.setPreferredSize(retD);
            preferredSize = retD;
        }
        return preferredSize;
    }

    private Dimension getItemSize(JMenu menu) {
        Dimension d = new Dimension(0, 0);
        int n = menu.getItemCount();
        for (int i = 0; i < n; i++) {
            Dimension itemD;
            JMenuItem item = menu.getItem(i);
            if (item instanceof JMenu) {
                itemD = getItemSize((JMenu) item);
            } else if (item != null) {
                itemD = item.getPreferredSize();
            } else {
                itemD = new Dimension(0, 0); // separator
            }
            d.width = Math.max(d.width, itemD.width);
            d.height = Math.max(d.height, itemD.height);
        }
        return d;
    }

    private static class ComboMenu extends JMenu {

        ArrowIcon iconRenderer;

        public ComboMenu(String label) {
            super(label);
            iconRenderer = new ArrowIcon(SwingConstants.SOUTH, true);
            setHorizontalTextPosition(JButton.LEFT);
            setFocusPainted(true);
        }

        @Override
        public void paintComponent(Graphics g) {
            super.paintComponent(g);
            Dimension d = this.getPreferredSize();
            int x = Math.max(0, d.width - d.height + (d.height- iconRenderer.getIconWidth())/2);
            int y = Math.max(0,(d.height- iconRenderer.getIconWidth())/2);
            iconRenderer.paintIcon(this, g, x, y);
        }
    }

    public static JMenu createMenu(String label) {
        return new ComboMenu(label);
    }
}

class ArrowIcon implements Icon, SwingConstants {

    private static final int DEFAULT_SIZE = 4;
    private int size;
    private int iconSize;
    private int direction;
    private boolean isEnabled;
    private BasicArrowButton iconRenderer;

    public ArrowIcon(int direction, boolean isEnabled) {
        this(DEFAULT_SIZE, direction, isEnabled);
    }

    public ArrowIcon(int iconSize, int direction, boolean isEnabled) {
        this.size = iconSize;
        this.iconSize = iconSize;
        this.direction = direction;
        this.isEnabled = isEnabled;
        iconRenderer = new BasicArrowButton(direction);
    }

    @Override
    public void paintIcon(Component c, Graphics g, int x, int y) {
        iconRenderer.paintTriangle(g, x, y, size, direction, isEnabled);
    }

    @Override
    public int getIconWidth() {
        switch (direction) {
            case NORTH:
            case SOUTH:
                return iconSize;
            case EAST:
            case WEST:
                return size;
        }
        return iconSize;
    }

    @Override
    public int getIconHeight() {
        switch (direction) {
            case NORTH:
            case SOUTH:
                return size;
            case EAST:
            case WEST:
                return iconSize;
        }
        return size;
    }
}
