/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or bbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * bbbuild/licenses/cddl-gpl-2-cp.txt.  This particular file is subject to
 * the "Classpath" exception as provided in the GPL Version 2 section of the
 * License file that accompanied this code. If applicable, add the following
 * below the License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 * Portions Copyrighted 2013 George Waldon
 */
package org.biobeans.modules.biotoolbar;

import java.util.prefs.PreferenceChangeListener;
import java.util.prefs.Preferences;
import org.openide.util.NbPreferences;

/**
 *
 * @author George Waldon
 */
public class QuerySettings {
    
    public static final String PROP_BIOINFORMATIONCENTER = "bioinformationcenter"; // NOI18N
    public static final String DEFAULT_CENTER = "NCBI"; // NOI18N

    public static final String PROP_SEARCH = "search"; // NOI18N
    public static final String DEFAULT_SEARCH = "PubMed"; // NOI18N
    
    private static QuerySettings querysettings = new QuerySettings();
    
    public static QuerySettings getDefault() {
        return querysettings;
    }
    
    private QuerySettings() {};
    
    public void addPreferenceChangeListener(PreferenceChangeListener settingListener) {
        prefs().addPreferenceChangeListener(settingListener);
    }

    public void removePreferenceChangeListener(PreferenceChangeListener settingListener) {
        prefs().removePreferenceChangeListener(settingListener);
    }
    
    /** Get name of the current bioinformation center.
     */
    public String getBioinformationCenter() {
        return prefs().get(PROP_BIOINFORMATIONCENTER, DEFAULT_CENTER);
    }
    
    public void setBioinformationcenter(String center) {
        prefs().put(PROP_BIOINFORMATIONCENTER, center);
    }
    
    /** Get name of the current bioinformation center.
     */
    public String getSearch() {
        return prefs().get(PROP_SEARCH, DEFAULT_SEARCH);
    }
    
    public void setSearch(String search) {
        prefs().put(PROP_SEARCH, search);
    }
    
    private Preferences prefs() {
        return NbPreferences.forModule(QuerySettings.class);
    }

    
}
