/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
 */
package org.biobeans.modules.bistro.activity;

import org.biobeans.modules.bistro.task.ActivityEvent;
import java.util.Set;

/**
 * Defines a single abstract or concrete invokable activity. Each BioService
 * contains at least one of these and may contain many.
 *
 * @param <ConfigurationType> the ConfigurationType associated with the
 * Activity. This is an arbitrary java class that provides details on how the
 * Activity is configured..
 */
public interface Activity<ConfigurationType> extends Configurable<ConfigurationType> {

    /**
     * Get the set of InputPort
     *
     * @return
     */
    public Set<Port> getInputPorts();

    /**
     * Get an InputPort byname
     *
     * @return the Port or null.
     */
    public Port getInputPort(String name);

    /**
     * Get the set of OutputPort
     *
     * @return
     */
    public Set<Port> getOutputPorts();

    /**
     * Get an OutputPort byname
     *
     * @return the Port or null.
     */
    public Port getOutputPort(String name);

    /**
     * Called when data has been received.
     *
     */
    public void notifyActivity(ActivityEvent de);

    
    @Override
    public void configure(ConfigurationType conf)
            throws ActivityConfigurationException;
}
