/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/
package org.biobeans.modules.bistro.activity;

import org.biobeans.modules.bistro.dataflow.DataPipe;

/** A pluggable point of data communication serving as input and output
 * of an activity.
 *
 */
public interface Port {

    /** The ID of the port.
     */
    public String getId();
    
    /** The MIME type of the port.
     * 
     */
    public String getType();
    
    /** The topological dimension of the data circulating through the port.
     * Values are -1 for undefined, 1 for atomic data, 2 for a list,
     * a collection or an array or a stream of data, etc.
     * 
     */
    public int getDepth();
    
    public void addPipe(DataPipe pipe);
    
    public boolean removePipe(DataPipe pipe);
    
}
 