/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/
package org.biobeans.modules.bistro.activity;

import org.biobeans.modules.bistro.task.ActivityEvent;
import java.util.ArrayList;
import java.util.List;
import org.biobeans.modules.bistro.dataflow.DataListener;
import org.biobeans.modules.bistro.dataflow.DataPipe;

/**
 *
 */
public class SimpleOutputPort extends SimplePort implements OutputPort {
    
    private final List<DataListener> dataListeners; 
    
    public SimpleOutputPort(String name, String type, int depth, Activity activity) {
        super(name, type, depth, activity);
        dataListeners = new ArrayList<>();
    }
    
    protected void addDataListener(DataListener l) { 
        synchronized (dataListeners) {
            dataListeners.add(l);
        }
    }
    
    protected void removeDataListener(DataListener l) {
        synchronized (dataListeners) {
            dataListeners.remove(l);
        }
    }
    
    @Override
    public void addPipe(DataPipe pipe) {
        super.addPipe(pipe);
        pipe.setSource(this);
        addDataListener(pipe);
    }

    @Override
    public void outputData(ActivityEvent de) {
        synchronized (dataListeners) {
            for (DataListener l : dataListeners) {
                l.data(de);
            }
        }
    }
}
