/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/
package org.biobeans.modules.bistro.activity;

import java.util.ArrayList;
import java.util.List;
import org.biobeans.modules.bistro.dataflow.DataPipe;

/**
 *
 */
public class SimplePort implements Port {

    /** The name of the port. */
    private final String id;
    
    /** The MIME type of the port. */
    private final String type;
    
    /** The depth of the port. */
    private final int depth;
    
    private final Activity activity;
    
    private final List<DataPipe> pipes = new ArrayList<>();

    public SimplePort(String id, String type, int depth, Activity activity) {
        this.id = id;
        this.type = type;
        this.depth = depth;
        this.activity = activity;
    }

    @Override
    public String getId() {
        return this.id;
    }
    
    @Override
    public String getType() {
        return this.type;
    }
    
    @Override
    public int getDepth() {
        return depth;
    }
    
    @Override
    public void addPipe(DataPipe pipe) {
        pipes.add(pipe);
    }
    
    @Override
    public boolean removePipe(DataPipe pipe) {
        return pipes.remove(pipe);
    }

    @Override
    public String toString() {
        return getClass().getSimpleName() + " " + getId();
    }
    
    protected Activity getActivity() {
        return activity;
    }
}
