/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/
package org.biobeans.modules.bistro.activity.localactivities;

import java.util.HashMap;
import java.util.Map;
import org.biobeans.modules.bistro.activity.AbstractActivity;
import org.biobeans.modules.bistro.activity.ActivityConfigurationException;
import org.biobeans.modules.bistro.activity.SimpleConfigurationBean;
import org.biobeans.modules.bistro.activity.SimpleInputPort;
import org.biobeans.modules.bistro.activity.SimpleOutputPort;
import org.biobeans.modules.bistro.data.DataContainer;
import org.biobeans.modules.bistro.data.factory.DataContainerFactory;
import org.biobeans.modules.bistro.task.CancelledByUserException;
import org.biobeans.modules.bistro.task.TaskExecutionException;
import org.openide.*;

/**
 *
 * @author George Waldon
 */
public class ManualIDInputActivity extends AbstractActivity<SimpleConfigurationBean> {
    
    private final String title;
    private final String label;
    
    public ManualIDInputActivity(String title, String label) {
        this.title = title;
        this.label = label;
    }
    
    private SimpleConfigurationBean configBean;

    @Override
    public SimpleConfigurationBean getConfiguration() {
        return this.configBean;
    }

    @Override
    public void configure(SimpleConfigurationBean configBean)
            throws ActivityConfigurationException {
        this.configBean = configBean;
        removeInputPorts();
        removeOutputPorts();
        addInputPort(new SimpleInputPort("fire","text/plain",0,this));
        addOutputPort(new SimpleOutputPort("id","text/plain",0,this));
    }

    @Override
    protected Map<String, DataContainer> execute(Map<String, DataContainer> inputMap) throws TaskExecutionException {
        String id = EnterIDDIalog();
        Map<String, DataContainer> iMap = new HashMap();
        if (id == null || id.length() == 0 || id.equals("")) {
            throw new CancelledByUserException();
        }
        DataContainer geneid = DataContainerFactory.create(id);
        iMap.put("id", geneid);
        return iMap;
    }
    
    private String EnterIDDIalog () {
        NotifyDescriptor.InputLine il = 
                new NotifyDescriptor.InputLine(label,title);

        Object ok = DialogDisplayer.getDefault ().notify (il);
        if (ok == NotifyDescriptor.OK_OPTION) {
            return il.getInputText().trim();

        }
        return "";
    }
}
