/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
 */
package org.biobeans.modules.bistro.data;

/**
 * A trivial interface to a service capable of providing unique identifiers on
 * demand. Used by the DataContainer object to 'fill in' any holes in its LSID map.
 *
 * LSID urn:lsid:<Authority>:<Namespace>:<ObjectID>[:<Version>]
 *
 * @author Tom Oinn
 * @author EMBL-EBI
 */
public interface LSIDProvider {

    public static final NamespaceEnumeration WFDEFINITION = new NamespaceEnumeration(
            "wfDefinition");

    public static final NamespaceEnumeration WFINSTANCE = new NamespaceEnumeration(
            "wfInstance");

    public static final NamespaceEnumeration DATACONTAINERLEAF = new NamespaceEnumeration(
            "dataItem");

    public static final NamespaceEnumeration DATACONTAINERCOLLECTION = new NamespaceEnumeration(
            "dataCollection");

    /**
     * Return a unique identifier to be used as an LSID or similar
     * @param namespace
     * @return 
     */
    public String getID(LSIDProvider.NamespaceEnumeration namespace);

    class NamespaceEnumeration {

        private String desc = "";

        public NamespaceEnumeration(String description) {
            this.desc = description;
        }

        @Override
        public String toString() {
            return this.desc;
        }
    }

}
