/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/
package org.biobeans.modules.bistro.dataflow;

import org.biobeans.modules.bistro.activity.Port;

/**
 * A link between a port and a DataListerner through which data flow. 
 * Data are always flowing from the source to the sink, from an output Port to 
 * a DataListerner, usually an InputPort. 
 */
public interface DataPipe extends DataListener {

    /**
     * Get the sink for events flowing through this pipe
     *
     * @return DataListener receiving events
     */
    public DataListener getSink();

    public void setSink(DataListener port);
    
    /**
     * Get the source for events flowing through this pipe
     *
     * @return output port generating events
     */
    public Port getSource();

    public void setSource(Port outputPort);
}
