/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/
package org.biobeans.modules.bistro.dataflow;

import org.biobeans.modules.bistro.task.ActivityEvent;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.biobeans.modules.bistro.activity.Port;

/**
 *
 */
public class SimpleDataPipe implements DataPipe {

    private static final Logger LOGGER = Logger.getLogger(SimpleDataPipe.class.getName());

    private Port source;

    private DataListener sink;
    
    /** The MIME type of the pipe.
     * 
     */
    private String type;
    
    protected void setType(String type) {
        if(type == null) {
            this.type = type;
        } else {
            if(!this.type.contentEquals(type)) {
                LOGGER.log(Level.SEVERE, "This pipe has mixed type: {0}", this.toString());
            }
        }
    }
	
    @Override
    public DataListener getSink() {
        return sink;
    }

    @Override
    public void setSink(DataListener port) {
        if(sink!=null) {
            throw new IllegalArgumentException("Attempt to add a second sink to a pipe!");
        }
        this.sink = port;
    }

    @Override
    public Port getSource() {
        return source;
    }
    
    @Override
    public void setSource(Port outputPort) {
        this.source = outputPort;
    }

    @Override
    public void data(ActivityEvent data) {
        ((DataListener) sink).data(data);
    }
    
}
