/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
 */
package org.biobeans.modules.bistro.io;

import java.io.IOException;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import org.apache.commons.httpclient.HostConfiguration;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.methods.PostMethod;
import org.biobeans.modules.bistro.data.Base64;
import org.biobeans.modules.bistro.data.DataContainer;

/**
 * This class uses an HTTP POST method to transmit data to a URL, and to return
 * the results in a map.
 *
 * @author Mark
 */
public class PostStreamTransmitter extends AbstractStreamTransmitter {

    /**
     *
     */
    @Override
    public Map<String, DataContainer> transmit(Map<String, String> map, StreamProcessor streamProcessor) throws TransmitterException {
        Map<String, DataContainer> outputMap = new HashMap();
        try {
            String path = context;
            path = (serviceName != null) ? path + "/" + serviceName : path;
            PostMethod method = new PostMethod(path);

            method.setFollowRedirects(true);

            Iterator dataIterator = map.keySet().iterator();

            if (dataIterator != null) {
                // Set Authentication Headers
                if ((userName != null) || (password != null)) {
                    method.addRequestHeader("Authorization", "Basic " + Base64.encodeString(userName + ":" + password));
                }

                // Set mime Headers
                if (mimeHeaders != null && mimeHeaders.keySet() != null) {
                    Iterator mimeIter = mimeHeaders.keySet().iterator();

                    while (mimeIter.hasNext()) {
                        String mimeParamName = (String) mimeIter.next();
                        method.addRequestHeader(mimeParamName, (String) mimeHeaders.get(mimeParamName));
                    }
                }

                // Set Data Params
                while (dataIterator.hasNext()) {
                    String paramName = (String) dataIterator.next();
                    String paramValue = (String) map.get(paramName);
                    paramValue = (paramValue == null) ? "" : paramValue;
                    method.addParameter(paramName, URLEncoder.encode(paramValue, "UTF-8"));
                }
            }

            HostConfiguration config = new HostConfiguration();
            config.setHost(host, port);
            HttpClient client = new HttpClient();
            client.setHostConfiguration(config);
            setProxy(client);
            client.executeMethod(method);

            outputMap = streamProcessor.processStream(method.getResponseBodyAsStream());

            int statusCode = method.getStatusCode();
            int statusLevel = statusCode / 100;

            // if there is an invalid response code from the service, throw a
            // TransmitterException
            if ((statusLevel != 2)) {
                TransmitterException ex = new TransmitterException(method.getStatusText());
                throw ex;
            }
        } catch (HttpException httpe) {
            throw createTxException(httpe);
        } catch (IOException ioe) {
            throw createTxException(ioe);
        } catch (IllegalArgumentException | TransmitterException th) {
            throw createTxException(th);
        }

        return outputMap;

    }

}
