/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
 */
package org.biobeans.modules.bistro.nodes;

import java.util.logging.Level;
import java.util.logging.Logger;
import org.netbeans.api.core.ide.ServicesTabNodeRegistration;
import org.biobeans.modules.bistro.services.BioService;
import org.biobeans.modules.bistro.services.BioServiceRegistry;
import org.biobeans.modules.bistro.services.BioServiceRoot;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataFolder;
import org.openide.nodes.Children;
import org.openide.util.Lookup;

public class RootServiceNode extends ServiceNode {

    private static final Logger LOGGER = Logger.getLogger(RootServiceNode.class.getName());

    private static RootServiceNode rootNode = null;

    @ServicesTabNodeRegistration(
            name = "BioServices", //NOI18N
            displayName = "org.biobeans.modules.bistro.services.Bundle#LBL_root", //NOI18N
            iconResource = "org/biobeans/modules/bistro/resources/rooticon.gif", //NOI18N
            position = 200
    )
    public static synchronized RootServiceNode getInstance() {
        if (rootNode == null) {
            BioServiceRegistry registry = Lookup.getDefault().lookup(BioServiceRegistry.class);
            DataFolder obj = findBioServiceRootFolder();
            rootNode = new RootServiceNode(registry.findService(BioServiceRoot.class.getName()), new ServiceNodeChildren(obj));
        }
        return rootNode;
    }

    /**
     * Returns "BioServices" folder.
     */
    private static DataFolder findBioServiceRootFolder() {
        FileObject fo = FileUtil.getConfigFile("BioServices"); //NOI18N
        if (fo == null) {
            IllegalStateException ex = new IllegalStateException();
            LOGGER.log(Level.WARNING, "BioServices folder not found in layer", ex); //NOI18N
            throw ex;
        }
        DataFolder df = DataFolder.findFolder(fo);
        return df;
    }

    private RootServiceNode(BioService service, Children ch) {
        super(service, ch);
    }

    @Override
    public boolean canRename() {
        return false;
    }

}
