/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
 */
package org.biobeans.modules.bistro.nodes;

import java.net.URL;
import java.util.LinkedList;
import java.util.logging.Logger;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;

/**
 *
 * @author George Waldon
 */
class ServiceNodeChildren<T> extends Children.Keys<DataObject> {

    private static final Logger LOGGER = Logger.getLogger(ServiceNodeChildren.class.getName());

    private final DataObject dto;
    private boolean initialized;

    public ServiceNodeChildren(DataObject dto) {
        this.dto = dto;
    }

    @Override
    protected void addNotify() {
        refreshKeys();
        initialized = true;
    }

    @Override
    protected void removeNotify() {
        setKeys(java.util.Collections.<DataObject>emptySet());
        initialized = false;
    }

    void refreshKeys() {
        final LinkedList<DataObject> keys = new LinkedList<>();
        if (dto instanceof DataFolder) {
            DataFolder dtf = (DataFolder) dto;
            DataObject[] objs = dtf.getChildren();
            for (DataObject obj : objs) {
                keys.add(obj);
            }
        } else {
            keys.add(dto);
        }
        setKeys(keys);
    }

    @Override
    protected Node[] createNodes(DataObject key) {
        Node n = null;
        if (key instanceof DataFolder) {
            FileObject fo = ((DataFolder)key).getPrimaryFile();
            Object icon = fo.getAttribute("iconBase");
            n = ServiceNodeFactory.DEFAULT_NODE_FACTORY.createServiceNodeFolder((DataFolder) key);
            if(icon!=null && (icon instanceof String) && (n instanceof AbstractNode)) {
                ((AbstractNode)n).setIconBaseWithExtension((String)icon);
            }
        } else {
            n = ServiceNodeFactory.DEFAULT_NODE_FACTORY.createServiceNode(key);
        }
        if (n != null) {
            return new Node[]{n};
        }
        LOGGER.info("A bioservice node is missing");
        return null;
    }

}
