/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
 */
package org.biobeans.modules.bistro.restservices;

import java.util.logging.Level;
import java.util.logging.Logger;
import org.biobeans.modules.bistro.activity.Activity;
import org.biobeans.modules.bistro.activity.ActivityConfigurationException;
import org.biobeans.modules.bistro.activity.SimpleConfigurationBean;
import org.biobeans.modules.bistro.activity.localactivities.ManualIDInputActivity;
import org.biobeans.modules.bistro.activity.rest.RESTActivity;
import org.biobeans.modules.bistro.activity.rest.RESTActivityConfigurationBean;
import org.biobeans.modules.bistro.data.DataContainer;
import org.biobeans.modules.bistro.data.factory.DataContainerFactory;
import org.biobeans.modules.bistro.dataflow.DataListener;
import org.biobeans.modules.bistro.dataflow.DataPipe;
import org.biobeans.modules.bistro.dataflow.SimpleDataPipe;
import org.biobeans.modules.bistro.services.BioService;
import org.biobeans.modules.bistro.task.ActivityEvent;
import org.biobeans.modules.bistro.task.ActivityTask;
import org.biobeans.modules.bistro.task.Counter;
import org.openide.util.NbBundle;

/**
 *
 * @author Georger Waldon
 */
public class UniProtFetchService extends RestService implements DataListener {

    private static final Logger LOGGER = Logger.getLogger(UniProtFetchService.class.getName());

    private static UniProtFetchService service;
    
    DataPipe inputPipe;

    public static BioService getDefault() {
        if (service == null) {
            service = new UniProtFetchService();
        }
        return service;
    }

    public UniProtFetchService() {
        initialize();
        setName(NbBundle.getMessage(UniProtFetchService.class, "LBL_UniprotFetchService"));
        setDescription(NbBundle.getMessage(UniProtFetchService.class, "LBL_fetchuniprot"));
    }

    private void initialize() {
        try {
            Activity inputID = new ManualIDInputActivity(getDialogLabel(),getDialogTitle());
            inputID.configure(new SimpleConfigurationBean(""));

            inputPipe = new SimpleDataPipe();
            inputID.getInputPort("fire").addPipe(inputPipe);

            DataPipe idPipe1 = new SimpleDataPipe();
            inputID.getOutputPort("id").addPipe(idPipe1);
            Activity upa = getActivity();
            upa.getInputPort("id").addPipe(idPipe1);

            DataPipe outputPipe = new SimpleDataPipe();
            upa.getOutputPort("responseBody").addPipe(outputPipe);
            outputPipe.setSink(this);

            //Get the id rfrom the ManualIDActivity
            DataPipe idPipe2 = new SimpleDataPipe();
            inputID.getOutputPort("id").addPipe(idPipe2);
            idPipe2.setSink(this); //TODO can only have one sink!!!
            
        } catch (ActivityConfigurationException ex) {
            LOGGER.log(Level.SEVERE, "Configuration exception in UniProt Fectch activity", ex);
        }
    }

    @Override
    public String getID() {
        return "bistro:rest:uniprot:proteinfetch";
    }
    
    String getDialogLabel() {
        return NbBundle.getMessage (UniProtFetchService.class, "LBL_fetchuniprot");
    }
    
    String getDialogTitle() {
        return NbBundle.getMessage (UniProtFetchService.class, "LBL_enterseqid");
    }

    protected Activity getActivity() throws ActivityConfigurationException {
        RESTActivity ac = new RESTActivity();
        RESTActivityConfigurationBean configbean = RESTActivityConfigurationBean.getDefaultInstance();
        configbean.setName(NbBundle.getMessage(UniProtFetchService.class, "LBL_uniprotfetch"));
        configbean.setUrlSignature("http://www.uniprot.org/uniprot/{id}.txt");
        ac.configure(configbean);
        addConfigurationBean(configbean);
        return ac;
    }

    @Override
    public ActivityTask execute() {
        final ActivityTask at = new ActivityTask();
        
        Runnable run = new Runnable() {
            @Override
            public void run() {
                DataContainer dc = DataContainerFactory.create("");
                inputPipe.data(new ActivityEvent(at,"fire",dc));
            }
            
        };
        Counter.pour(run);
        return at;
    }
    
    @Override
    public void data(ActivityEvent event) {
        ActivityTask at = event.getTask();
        at.notifyActivity(event);
    }
    
}
