/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/
package org.biobeans.modules.bistro.services;

import javax.swing.event.ChangeListener;
import org.biobeans.modules.bistro.task.ActivityTask;
import org.openide.nodes.Node;

/** The BioService is the basic functional unit within Bistro service offering. 
 * BioService are centralized in NetBeans xml filesystem where they can be 
 * looked up upon and instantiated. Each BioService contains one or many 
 * activities connected together in a non-editable workflow and 
 * provides varied operations on data involving outside resources and computations.
 *
 * @author George Waldon
 */
public interface BioService {

    /** Unique ID. A given ID used to retrieve the BiooService in the registry.
     * 
     * @return The ID. 
     */
    public String getID();

    /** A human friendly name for this service.
     *
     * @return The name.
     */
    public String getName();

    /** A human friendly description of this service.
     *
     * @return The description.
     */
    public String getDescription();

    /* Get icon base with extension.
     */
    public String getIconBase();
    
    /** The simplest way to execute a service.
     *
     *  @return An ActivityTask to follow this resulting activities.
     */
    public ActivityTask execute();

    public void addChangeListener(ChangeListener listener);

    public void removeChangeListener(ChangeListener listener);

    /** Getter for properties associated with the service.
    */
    public Node.PropertySet[] getPropertySets();

}
