/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
 */
package org.biobeans.modules.bistro.services;

import java.util.logging.Logger;
import javax.swing.event.ChangeListener;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.biobeans.modules.bistro.task.ActivityTask;
import org.openide.nodes.Node;

/** A root for all BioService interesting for tree node representation.
 *
 * @author George Waldon
 */
public class BioServiceRoot implements BioService {

    private static final Logger LOGGER = Logger.getLogger(BioServiceRoot.class.getName());

    private static BioService rootservice;

    public static BioService getDefault() {
        if (rootservice == null) {
            rootservice = new BioServiceRoot();
        }
        return rootservice;
    }

    private BioServiceRoot() {
        BioServiceRegistry registry = Lookup.getDefault().lookup(BioServiceRegistry.class);
        registry.register(this);
    }

    @Override
    public String getID() {
        return "root"; //NOI18N
    }

    @Override
    public String getName() {
        return NbBundle.getMessage(BioServiceRoot.class, "LBL_root");
    }

    @Override
    public String getDescription() {
        return NbBundle.getMessage(BioServiceRoot.class, "DES_root");
    }

    @Override
    public void addChangeListener(ChangeListener listener) {
        //do nothing
    }

    @Override
    public void removeChangeListener(ChangeListener listener) {
        //do nothing
    }

    @Override
    public String getIconBase() {
        return NbBundle.getMessage(BioServiceRoot.class, "ICO_root");
    }

    @Override
    public ActivityTask execute() {
        //Do nothing
        return new ActivityTask();
    }   

    @Override
    public Node.PropertySet[] getPropertySets() {
        return new Node.PropertySet[0];
    }

}
