/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
 */
package org.biobeans.modules.bistro.services;

import org.openide.util.Lookup;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;
import org.openide.util.lookup.ProxyLookup;

/**
 *
 * @author George Waldon
 */
class ServiceLookup extends ProxyLookup {

    private static InstanceContent instanceContent = new InstanceContent();
    private static Lookup instanceLookup = new AbstractLookup(instanceContent);

    public ServiceLookup() {
        super(new Lookup[]{
            instanceLookup
        });
    }

    /**
     * Register new instance.
     */
    public void register(Object obj) {
        instanceContent.add(obj);
    }

    /**
     * Register new instance.
     *
     * @param obj source
     * @param conv convertor which will postpone instantiation
     */
    public <T, R> void register(T obj, InstanceContent.Convertor<T, R> conv) {
        instanceContent.add(obj, conv);
    }

    /**
     * Unregisters the service.
     */
    public void unregister(Object obj) {
        instanceContent.remove(obj);
    }

    /**
     * Unregisters the service registered with a convertor.
     */
    public <T, R> void unregister(T obj, InstanceContent.Convertor<T, R> conv) {
        instanceContent.remove(obj, conv);
    }
}
