/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
 */
package org.biobeans.modules.bistro.task;

import java.util.logging.Logger;
import org.openide.util.RequestProcessor;


/** A place to get served by Bistro
 *
 * @author George Waldon
 */
public class Counter {
    
    private static final Logger LOGGER = Logger.getLogger(Counter.class.getName());
    
    private static volatile Counter instance;
    
    private static final RequestProcessor RP_BARMAN = new RequestProcessor("barman", 1);

    private static final RequestProcessor RP_PERCOLATING = new RequestProcessor("percolator", 100);

    public static Counter getDefault() {
        if(instance==null)
            instance = new Counter();
        return instance;
    }
    
    /** Run a task with a throughput of 1.
     * 
     * @param r A Runnable to run.
     */
    public static void pour(Runnable r) {
        RP_BARMAN.post(r);
    }
    
    /** Run a task with a throughput of many.
     * 
     * @param r A Runnable to run.
     */
    public static void percolate (Runnable r) {
        RP_PERCOLATING.post(r);
    }

}
