/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
 */
package org.biobeans.modules.bistro.activity.localactivities.biojava;

import java.util.HashMap;
import java.util.Map;
import org.biobeans.modules.bistro.data.DataContainer;
import org.biobeans.modules.bistro.data.DataContainerAdapter;
import org.biojavax.bio.seq.RichSequence;
import org.junit.After;
import static org.junit.Assert.assertTrue;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.assertTrue;

/**
 *
 * @author George Waldon
 */
public class GenBankToBiojavaActivityTest {
    
        String interferon ="LOCUS       HUMINTERFE               588 bp    mRNA    linear   PRI 04-MAY-1994\n" +
"DEFINITION  Human interferon mRNA, complete cds.\n" +
"ACCESSION   L25664\n" +
"VERSION     L25664.1  GI:479010\n" +
"KEYWORDS    interferon.\n" +
"SOURCE      Homo sapiens (human)\n" +
"  ORGANISM  Homo sapiens\n" +
"            Eukaryota; Metazoa; Chordata; Craniata; Vertebrata; Euteleostomi;\n" +
"            Mammalia; Eutheria; Euarchontoglires; Primates; Haplorrhini;\n" +
"            Catarrhini; Hominidae; Homo.\n" +
"REFERENCE   1  (bases 1 to 588)\n" +
"  AUTHORS   Whaley,A.E., Meka,C.S., Harbison,L.A., Hunt,J.S. and Imakawa,K.\n" +
"  TITLE     Identification and cellular localization of unique interferon mRNA\n" +
"            from human placenta\n" +
"  JOURNAL   J. Biol. Chem. 269 (14), 10864-10868 (1994)\n" +
"   PUBMED   7511610\n" +
"COMMENT     Original source text: Homo sapiens placenta cDNA to mRNA.\n" +
"FEATURES             Location/Qualifiers\n" +
"     source          1..588\n" +
"                     /organism=\"Homo sapiens\"\n" +
"                     /mol_type=\"mRNA\"\n" +
"                     /db_xref=\"taxon:9606\"\n" +
"                     /cell_line=\"JM101\"\n" +
"                     /cell_type=\"trophoblast\"\n" +
"                     /tissue_type=\"placenta\"\n" +
"     CDS             1..588\n" +
"                     /note=\"disulfide bond 1..99 disulfide bond 29..139;\n" +
"                     putative\"\n" +
"                     /codon_start=1\n" +
"                     /product=\"interferon\"\n" +
"                     /protein_id=\"AAA36123.1\"\n" +
"                     /db_xref=\"GI:479011\"\n" +
"                     /translation=\"MAFVLSLLMALVLVSYGPGGSLGCDLSQNHVLVGRKNLRLLDEM\n" +
"                     RRLSPHFCLQDRKDFALPQEMVEGGQLQEAQAISVLHEMLQQSFNLFHTEHSSAAWDT\n" +
"                     TLLEPCRTGLHQQLDNLDACLGQVMGEEDSALGRTGPTLALKRYFQGIHVYLKEKGYS\n" +
"                     DCAWETVRLEIMRSFSSLISLQERLRMMDGDLSSP\"\n" +
"ORIGIN      \n" +
"        1 atggccttcg tgctctctct actcatggcc ctggtgctgg tcagctacgg cccaggagga\n" +
"       61 tccctgggtt gtgacctgtc tcagaaccac gtgctggttg gcaggaagaa cctcaggctc\n" +
"      121 ctggacgaaa tgaggagact ctcccctcac ttttgtctgc aggacagaaa agacttcgct\n" +
"      181 ttaccccagg aaatggtgga gggcggccag ctccaggagg cccaggccat ctctgtgctc\n" +
"      241 catgagatgc tccagcagag cttcaacctc ttccacacag agcactcctc tgctgcctgg\n" +
"      301 gacaccaccc tcctggagcc atgccgcact ggactccatc agcagctgga caacctggat\n" +
"      361 gcctgcctgg ggcaggtgat gggagaggaa gactctgccc tgggaaggac gggccccacc\n" +
"      421 ctggctctga agaggtactt ccagggcatc catgtctacc tgaaagagaa gggatacagc\n" +
"      481 gactgcgcct gggaaaccgt cagactggaa atcatgagat ccttctcttc attaatcagc\n" +
"      541 ttgcaagaaa ggttaagaat gatggatgga gacctgagct caccttga\n" +
"//";
        
    public GenBankToBiojavaActivityTest() {
    }
    
    @Before
    public void setUp() {
    }
    
    @After
    public void tearDown() {
    }

    @Test
    public void testActivity() throws Exception {
        Map<String, DataContainer> inputMap = new HashMap<>();
        DataContainerAdapter inAdapter = new DataContainerAdapter(inputMap);
        inAdapter.putString("inputText", interferon);
        
        GenBankToBiojavaActivity ac = new GenBankToBiojavaActivity();
        GenBankToBiojavaActivityConfigurationBean config = 
                new GenBankToBiojavaActivityConfigurationBean();
        config.setProteinParser(false);
        ac.configure(config);
        Map<String, DataContainer> outputMap = ac.execute(inputMap);
        DataContainerAdapter outAdapter = new DataContainerAdapter(outputMap);
        RichSequence rseq = (RichSequence)outAdapter.getSerializable("outputSeq");
        
        assertTrue(rseq.length()==588);               
        
    }
    
}
