/*
 *                    BioJava development code
 *
 * This code may be freely distributed and modified under the
 * terms of the GNU Lesser General Public Licence.  This should
 * be distributed with the code.  If you do not have a copy,
 * see:
 *
 *      http://www.gnu.org/copyleft/lesser.html
 *
 * Copyright for this code is held jointly by the individual
 * authors.  These should be listed in @author doc comments.
 *
 * For more information on the BioJava project and its aims,
 * or to join the biojava-l mailing list, visit the home page
 * at:
 *
 *      http://www.biojava.org/
 *
 */
package org.biobeans.demo.biojava;

import java.awt.event.ActionEvent;
import org.biobeans.demo.biojava.bj1.blastxml.BlastParser;
import org.biobeans.demo.biojava.bj1.circularlocationtester.TestFrame;
import org.biobeans.demo.biojava.bj1.eventbasedparsing.Blast2HTML;
import org.biobeans.demo.biojava.bj1.alignment.SmithWatermanExample;
import org.biobeans.demo.biojava.bj1.seqviewer.BeadDemo;
import org.biobeans.demo.biojava.bj1.seqviewer.EmblViewer2;
import org.biobeans.demo.biojava.bj1.seqviewer.TraceViewer;

/**
 *
 * @author George Waldon
 */
public class Actions {

    public static class BlastParserAction extends DemoAction {

        public BlastParserAction() {
            super("Blast Parser");
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            Runnable run = new BlastParser();
            runAsynchronously(run);
        }
    }

    public static class CircularLocationTestAction extends DemoAction {

        public CircularLocationTestAction() {
            super("Test Circular Location");
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            DemoTopComponent dtc = findDemoComponent();
            dtc.setPanel(new TestFrame());
            dtc.open();
            dtc.requestActive();
        }
    }

    public static class Blast2HTMLAction extends DemoAction {

        public Blast2HTMLAction() {
            super("Blast2HTML");
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            Runnable run = new Blast2HTML();
            runAsynchronously(run);
        }
    }
    
    public static class BeadDemoAction extends DemoAction {

        public BeadDemoAction() {
            super("Bead Demo");
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            DemoTopComponent dtc = findDemoComponent();
            BeadDemo db = new BeadDemo();
            dtc.setPanel(db);
            dtc.open();
            dtc.requestActive();
            runAsynchronously(db);
        }
    }

    public static class TraceViewerAction extends DemoAction {

        public TraceViewerAction() {
            super("Trace Viewer");
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            DemoTopComponent dtc = findDemoComponent();
            TraceViewer tv = new TraceViewer();
            dtc.setPanel(tv);
            dtc.open();
            dtc.requestActive();
            runAsynchronously(tv);
        }
    }

    public static class EmblViewer2Action extends DemoAction {

        public EmblViewer2Action() {
            super("EMBL Viewer");
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            DemoTopComponent dtc = findDemoComponent();
            EmblViewer2 ev = new EmblViewer2();
            dtc.setPanel(ev);
            dtc.open();
            dtc.requestActive();
            runAsynchronously(ev);
        }
    }

    public static class SmithWatermanAction extends DemoAction {

        public SmithWatermanAction() {
            super("Legacy SmithWaterman");
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            Runnable run = new SmithWatermanExample();
            runAsynchronously(run);
        }
    }
}
