/*
 *                    BioJava development code
 *
 * This code may be freely distributed and modified under the
 * terms of the GNU Lesser General Public Licence.  This should
 * be distributed with the code.  If you do not have a copy,
 * see:
 *
 *      http://www.gnu.org/copyleft/lesser.html
 *
 * Copyright for this code is held jointly by the individual
 * authors.  These should be listed in @author doc comments.
 *
 * For more information on the BioJava project and its aims,
 * or to join the biojava-l mailing list, visit the home page
 * at:
 *
 *      http://www.biojava.org/
 *
 */
package org.biobeans.demo.biojava;

import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.openide.cookies.OpenCookie;
import org.openide.cookies.ViewCookie;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;

/**
 *
 * @author gwaldon
 */
public class DemUtils {

    private static final Logger LOGGER = Logger.getLogger(DemUtils.class.getName());

    private DemUtils(){};

    public static FileObject getDemoFile(String name) {
        //Retrieve user directory and create biojava demo folder
        String s = System.getProperty("user.home"); // NOI18N
        File home = new File(s);
        home = FileUtil.normalizeFile(home);
        FileObject homeDir = FileUtil.toFileObject(home);
        FileObject desfolder = null;
        FileObject oFile = null;
        try {
            desfolder = FileUtil.createFolder(homeDir, "biobeans/demo/biojava"); // NOI18N
            if(desfolder==null) return null;
            oFile = FileUtil.createData(desfolder, name);
        } catch (IOException ex) {
            LOGGER.log(Level.INFO, "", ex); //NOI18N
        }
        return oFile;
    }

    public static void viewFile(FileObject oFile) {
        DataObject dob = null;
        try {
            dob = DataObject.find(oFile);
        } catch (DataObjectNotFoundException ex) {
            LOGGER.log(Level.INFO, "", ex);
            return;
        }
        OpenCookie oc = dob.getLookup().lookup(OpenCookie.class);
        if (oc != null) {
            oc.open();
        }
        ViewCookie vc = dob.getLookup().lookup(ViewCookie.class);
        if (vc != null) {
            vc.view();
        }
    }
}
