/*
 *                    BioJava development code
 *
 * This code may be freely distributed and modified under the
 * terms of the GNU Lesser General Public Licence.  This should
 * be distributed with the code.  If you do not have a copy,
 * see:
 *
 *      http://www.gnu.org/copyleft/lesser.html
 *
 * Copyright for this code is held jointly by the individual
 * authors.  These should be listed in @author doc comments.
 *
 * For more information on the BioJava project and its aims,
 * or to join the biojava-l mailing list, visit the home page
 * at:
 *
 *      http://www.biojava.org/
 *
 */

package org.biobeans.demo.biojava;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.lang.ref.WeakReference;
import java.util.logging.Logger;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.UIManager;
import org.openide.util.Lookup;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;

/**
 *
 * @author George Waldon
 */
public class DemoTopComponent extends TopComponent {

    static public final Color SCREENCOLORBACKGROUND = new Color(235, 239, 249);
    static public final int START_PAGE_MIN_WIDTH = 600;
    static public final int STARTPANEL_MIN_WIDTH = 600;
    static public final int STARTPANEL_MAX_WIDTH = 800;

    //Fonts
    static public final int FONT_SIZE = getDefaultFontSize();

    static int getDefaultFontSize() {
        Integer customFontSize = (Integer)UIManager.get("customFontSize"); // NOI18N
        if (customFontSize != null) {
            return customFontSize.intValue();
        } else {
            Font systemDefaultFont = UIManager.getFont("TextField.font"); // NOI18N
            return (systemDefaultFont != null)
                ? systemDefaultFont.getSize()
                : 12;
        }
    }

    private static final Logger LOGGER = Logger.getLogger(DemoTopComponent.class.getName());

    static final long serialVersionUID=1L;

    private JComponent panel;
    private JComponent testpane;
    private static WeakReference<DemoTopComponent>  component = new WeakReference<DemoTopComponent>(null);
    private static final String PREFERRED_ID = "bjdemo"; //NOI18N
    private boolean initialized = false;

    private DemoTopComponent(){
        associateLookup(Lookup.EMPTY);
        setLayout(new BorderLayout());
        setName("BioJava Demo"); //NOI18N
        panel = null;
        initialized = false;
    }

    public void setPanel(JPanel pane) {
        if(panel!=null) {
            remove(panel);
            panel = null;
            revalidate();
        }
        testpane = pane;
        if(initialized)
            Initialize();
    }

    @Override
    protected String preferredID() {
        return PREFERRED_ID;
    }

    private void Initialize() {
        if( null == panel ) {
            JScrollPane scroll = new JScrollPane(testpane);
            scroll.setBorder(BorderFactory.createEmptyBorder());
            scroll.getViewport().setOpaque(false);
            scroll.setOpaque(false);
            scroll.getViewport().setPreferredSize(new Dimension(START_PAGE_MIN_WIDTH,100));
            JScrollBar vertical = scroll.getVerticalScrollBar();
            if( null != vertical ) {
                vertical.setBlockIncrement(30*FONT_SIZE);
                vertical.setUnitIncrement(FONT_SIZE);
            }
            panel = scroll;
            add( panel, BorderLayout.CENTER );
            setFocusable( false );
        }
    }

    public static synchronized DemoTopComponent findInstance() {
        DemoTopComponent dtc = component.get();
        if (dtc == null) {
            TopComponent comp = WindowManager.getDefault().findTopComponent(PREFERRED_ID); // NOI18N
            if (comp != null) {
                if (comp instanceof DemoTopComponent) {
                    dtc = (DemoTopComponent) comp;
                    component = new WeakReference<DemoTopComponent>(dtc);
                } else {
                    LOGGER.info("Unexcepted DemoTopComponent component found");  //NOI18N
                    dtc = DemoTopComponent.createDemoTopComponent();
                }
            } else {
                dtc = DemoTopComponent.createDemoTopComponent();
            }
        }
        return dtc;
    }

    public static DemoTopComponent createDemoTopComponent() {
        DemoTopComponent dtc = component.get();
        if(dtc == null) {
            dtc = new DemoTopComponent();
            component = new WeakReference<DemoTopComponent>(dtc);
        }
        return dtc;
    }

    @Deprecated
    @Override
    public void requestFocus(){
        if( panel!=null )
            panel.requestFocus();
    }

    @Deprecated
    @Override
    public boolean requestFocusInWindow(){
        if( panel!=null )
            return panel.requestFocusInWindow();
        return super.requestFocusInWindow();
    }

    @Override
    public void requestActive() {
        super.requestActive();
        if( panel!=null )
            panel.requestFocus();
    }

    @Override
    public void addNotify() {
        if (!initialized) {
            initialized = true;
            Initialize();
        }
        super.addNotify();
    }

    @Override
    protected void componentShowing() {
        if (!initialized) {
            initialized = true;
            Initialize();
        }
        super.componentShowing();
    }

    @Override
    public int getPersistenceType() {
        return TopComponent.PERSISTENCE_NEVER;
    }

}

