package org.biobeans.demo.biojava.bj1.blastxml;

import java.io.*;
import java.net.URL;
import java.net.URLConnection;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.biojava.bio.program.ssbind.*;
import org.biojava.bio.search.*;
import org.biojava.bio.program.sax.blastxml.BlastXMLParserFacade;
import org.biojava.bio.seq.db.*;
import org.xml.sax.*;
import org.biojava.bio.*;
import org.openide.windows.InputOutput;
import org.openide.windows.IOProvider;

public class BlastParser implements Runnable {

    private static final Logger LOGGER = Logger.getLogger(BlastParser.class.getName());

    public void run() {
        try {
            //get the Blast input as a Stream
            URL url = BlastParser.class.getResource("input.xml");
            URLConnection uc = url.openConnection();
            InputStreamReader is = new InputStreamReader(uc.getInputStream());

            //make a BlastLikeSAXParser
            BlastXMLParserFacade parser = new BlastXMLParserFacade();

            //make the SAX event adapter that will pass events to a Handler.
            SeqSimilarityAdapter adapter = new SeqSimilarityAdapter();

            //set the parsers SAX event adapter
            parser.setContentHandler(adapter);

            //The list to hold the SeqSimilaritySearchResults
            List results = new ArrayList();

            //create the SearchContentHandler that will build SeqSimilaritySearchResults
            //in the results List
            SearchContentHandler builder = new BlastLikeSearchBuilder(results,
                    new DummySequenceDB("queries"), new DummySequenceDBInstallation());

            //register builder with adapter
            adapter.setSearchContentHandler(builder);

            //parse the file, after this the result List will be populated with
            //SeqSimilaritySearchResults
            parser.parse(new InputSource(is));

            InputOutput io = IOProvider.getDefault().getIO("BlastParser", false);
            io.select();
            io.setFocusTaken(false);
            final PrintWriter pw = io.getOut();

            try {
                //output some blast details
                for (Iterator i = results.iterator(); i.hasNext();) {
    //        pw.println("loop");
                    SeqSimilaritySearchResult result =
                            (SeqSimilaritySearchResult) i.next();

                    Annotation anno = result.getAnnotation();

                    for (Iterator j = anno.keys().iterator(); j.hasNext();) {
                        Object key = j.next();
                        Object property = anno.getProperty(key);
                        pw.println(key + " : " + property);
                    }
                    pw.println("Hits: ");

                    //list the hits
                    for (Iterator k = result.getHits().iterator(); k.hasNext();) {
                        SeqSimilaritySearchHit hit =
                                (SeqSimilaritySearchHit) k.next();
                        pw.print("\tmatch: " + hit.getSubjectID());
                        pw.println("\te score: " + hit.getEValue());
                    }

                    pw.println("\n");
                }
            }
            finally {
                pw.close();
            }
            
        } catch (SAXException ex) {
            //XML problem
            LOGGER.log(Level.INFO, null, ex);
        } catch (IOException ex) {
            //IO problem, possibly file not found
            LOGGER.log(Level.INFO, null, ex);
        } catch (BioException be) {
            LOGGER.log(Level.INFO, null, be);
        }
    }
}
