/*
 *                  BioJava development code
 *
 * This code may be freely distributed and modified under the
 * terms of the GNU Lesser General Public Licence.  This should
 * be distributed with the code.  If you do not have a copy,
 * see:
 *
 *      http://www.gnu.org/copyleft/lesser.html
 *
 * Copyright for this code is held jointly by the individual
 * authors.  These should be listed in @author doc comments.
 *
 * For more information on the BioJava project and its aims,
 * or to join the biojava-l mailing list, visit the home page
 * at:
 *
 */
package org.biobeans.demo.biojava.bj4.alignment;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.PrintWriter;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import org.biojava.nbio.alignment.Alignments;
import org.biojava.nbio.alignment.Alignments.PairwiseSequenceAlignerType;
import org.biojava.nbio.alignment.SimpleGapPenalty;
import org.biojava.nbio.core.alignment.matrices.SimpleSubstitutionMatrix;
import org.biojava.nbio.core.alignment.template.Profile;
import org.biojava.nbio.core.alignment.template.SequencePair;
import org.biojava.nbio.core.alignment.template.SubstitutionMatrix;
import org.biojava.nbio.core.sequence.ProteinSequence;
import org.biojava.nbio.core.sequence.compound.AminoAcidCompound;
import org.biojava.nbio.core.sequence.io.FastaReaderHelper;
import org.biojava.nbio.core.util.ConcurrencyTools;
import org.openide.awt.ActionRegistration;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionReferences;
import org.openide.awt.ActionID;
import org.openide.modules.InstalledFileLocator;
import org.openide.util.NbBundle.Messages;
import org.openide.util.RequestProcessor;
import org.openide.windows.IOProvider;
import org.openide.windows.InputOutput;

@ActionID(category = "Biology",
id = "org.biobeans.demo.biojava.bj4.alignment.MultipleAlignmentAction")
@ActionRegistration(displayName = "#CTL_MultipleAlignmentAction")
@ActionReferences({
    @ActionReference(path = "Menu/Demos/alignment", position = 300)
})
@Messages("CTL_MultipleAlignmentAction=Multiple Alignment")
public final class MultipleAlignmentAction implements ActionListener {

    @Override
    public void actionPerformed(ActionEvent e) {
        Runnable r = new Runnable() {

            @Override
            public void run() {
                InputOutput io = IOProvider.getDefault().getIO("Alignment", false);
                io.select();
                io.setFocusTaken(false);
                PrintWriter pw = io.getOut();

                try {
                    
                    String sfile1 = "Q21691";
                    String sfile2 = "Q21495";
                    String sfile3 = "O48771";
                    File file1 = InstalledFileLocator.getDefault().locate("files/alignment/"+sfile1+".fasta", "", true);
                    File file2 = InstalledFileLocator.getDefault().locate("files/alignment/"+sfile2+".fasta", "", true);
                    File file3 = InstalledFileLocator.getDefault().locate("files/alignment/"+sfile3+".fasta", "", true);
                    List<ProteinSequence> lst = new ArrayList<>();
                    lst.add(getSequenceForFile(sfile1,file1,pw));
                    lst.add(getSequenceForFile(sfile2,file2,pw));
                    lst.add(getSequenceForFile(sfile3,file3,pw));
                    
//                    String[] ids = new String[]{"Q21691", "Q21495", "O48771"};
//                    List<ProteinSequence> lst = new ArrayList<ProteinSequence>();
//                    for (String id : ids) {
//                        lst.add(getSequenceForId(id, pw));
//                    }

                    pw.println();
                    pw.println("MULTIPLE ALIGNMENT");
                    multipleSequenceAlignment(lst, pw);
                } catch (Exception ex) {
                } finally {
                    pw.close();
                }
            }
        };
        RequestProcessor.getDefault().post(r);
    }
    
    private void alignPairGlobal(ProteinSequence s1, ProteinSequence s2, PrintWriter pw) throws Exception {

        SubstitutionMatrix<AminoAcidCompound> matrix = SimpleSubstitutionMatrix.getBlosum62();
        SequencePair<ProteinSequence, AminoAcidCompound> pair = Alignments.getPairwiseAlignment(s1, s2,
                PairwiseSequenceAlignerType.GLOBAL, new SimpleGapPenalty(), matrix);
        pw.printf("%n%s vs %s%n%s", pair.getQuery().getAccession(), pair.getTarget().getAccession(), pair);
    }
    
     private void alignPairLocal(ProteinSequence s1, ProteinSequence s2, PrintWriter pw) throws Exception {
        SubstitutionMatrix<AminoAcidCompound> matrix = SimpleSubstitutionMatrix.getBlosum62();
        SequencePair<ProteinSequence, AminoAcidCompound> pair = Alignments.getPairwiseAlignment(s1, s2,
                PairwiseSequenceAlignerType.LOCAL, new SimpleGapPenalty(), matrix);
        pw.printf("%n%s vs %s%n%s", pair.getQuery().getAccession(), pair.getTarget().getAccession(), pair);
    }
 
      private static void multipleSequenceAlignment(List<ProteinSequence> lst, PrintWriter pw) throws Exception {
        Profile<ProteinSequence, AminoAcidCompound> profile = Alignments.getMultipleSequenceAlignment(lst);
        pw.printf("Clustalw:%n%s%n", profile);
        ConcurrencyTools.shutdown();
    }
      
    private ProteinSequence getSequenceForId(String uniProtId,PrintWriter pw) throws Exception {
        URL uniprotFasta = new URL(String.format("http://www.uniprot.org/uniprot/%s.fasta", uniProtId));
        ProteinSequence seq = FastaReaderHelper.readFastaProteinSequence(uniprotFasta.openStream()).get(uniProtId);
        pw.printf("id : %s %s%n%s%n", uniProtId, seq, seq.getOriginalHeader());
        return seq;
    }
    
    private ProteinSequence getSequenceForFile(String sfile, File file,PrintWriter pw) throws Exception {
        ProteinSequence seq = FastaReaderHelper.readFastaProteinSequence(file).get(sfile);
        pw.printf("id : %s %s%n%s%n", sfile, seq, seq.getOriginalHeader());
        return seq;
    }
}
