/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
 */
package org.biobeans.modules.bistro.activity.onlineactivities.ncbi;

import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;
import org.biobeans.modules.bistro.activity.ActivityConfigurationException;
import org.biobeans.modules.bistro.activity.SimpleInputPort;
import org.biobeans.modules.bistro.data.DataContainer;
import org.biobeans.modules.bistro.data.DataContainerAdapter;
import org.biobeans.modules.bistro.io.TransmitterException;
import org.biobeans.modules.bistro.task.TaskExecutionException;

/**
 * This processor returns a GB Seq formatted xml record.
 *
 *  GBSeq ::= SEQUENCE {
 *    locus VisibleString OPTIONAL ,
 *    length INTEGER ,
 *    strandedness VisibleString OPTIONAL ,
 *    moltype VisibleString ,
 *   topology VisibleString OPTIONAL ,
 *   division VisibleString OPTIONAL ,
 *   update-date VisibleString OPTIONAL ,
 *   create-date VisibleString OPTIONAL ,
 *   update-release VisibleString OPTIONAL ,
 *   create-release VisibleString OPTIONAL ,
 *   definition VisibleString OPTIONAL ,
 *   primary-accession VisibleString OPTIONAL ,
 *   entry-version VisibleString OPTIONAL ,
 *   accession-version VisibleString OPTIONAL ,
 *   other-seqids SEQUENCE OF GBSeqid OPTIONAL ,
 *   secondary-accessions SEQUENCE OF GBSecondary-accn OPTIONAL,
 *   project VisibleString OPTIONAL ,
 *   keywords SEQUENCE OF GBKeyword OPTIONAL ,
 *   segment VisibleString OPTIONAL ,
 *   source VisibleString OPTIONAL ,
 *   organism VisibleString OPTIONAL ,
 *   taxonomy VisibleString OPTIONAL ,
 *   references SEQUENCE OF GBReference OPTIONAL ,
 *   comment VisibleString OPTIONAL ,
 *   comment-set SEQUENCE OF GBComment OPTIONAL ,
 *   struc-comments SEQUENCE OF GBStrucComment OPTIONAL ,
 *   primary VisibleString OPTIONAL ,
 *    source-db VisibleString OPTIONAL ,
 *   database-reference VisibleString OPTIONAL ,
 *   feature-table SEQUENCE OF GBFeature OPTIONAL ,
 *   feature-set SEQUENCE OF GBFeatureSet OPTIONAL ,
 *   sequence VisibleString OPTIONAL ,  -- Optional for contig, wgs, etc.
 *   contig VisibleString OPTIONAL ,
 *   alt-seq SEQUENCE OF GBAltSeqData OPTIONAL ,
 *   xrefs SEQUENCE OF GBXref OPTIONAL
 * }
 * 
 * @author mfortner
 *
 * @tavinput id The nucleotide accession.
 * @tavoutput outputText a GB Seq formatted record
 */
public class NucleotideGBSeqActivity extends AbstractEFetchActivity<NucleotideGBSeqActivityConfigurationBean> {

    private static Logger LOGGER = Logger.getLogger(NucleotideGBSeqActivity.class.getName());

    private NucleotideGBSeqActivityConfigurationBean configbean;
    
    public NucleotideGBSeqActivity() {
    }

    @Override
    public void configure(NucleotideGBSeqActivityConfigurationBean conf) throws ActivityConfigurationException {
        configbean = conf;
        configureEFetchActivity(configbean);
        addInputPort(new SimpleInputPort("id","text/plain",0,this));
    }

    @Override
    public NucleotideGBSeqActivityConfigurationBean getConfiguration() {
        return configbean;
    }

    @Override
    public Map<String, DataContainer> execute(Map<String, DataContainer> inputMap) throws TaskExecutionException {
        DataContainerAdapter inAdapter = new DataContainerAdapter(inputMap);
        this.id = inAdapter.getString("id");

        Map<String, String> transmitterMap = new HashMap<>();
        transmitterMap.put("id", this.id);
        transmitterMap.put("db", this.db);
        transmitterMap.put("rettype", this.rettype);
        transmitterMap.put("retmode", this.retmode);

        Map<String, DataContainer> outputMap;
        try {
            outputMap = this.transmit(transmitterMap);
        } catch (MalformedURLException | TransmitterException e) {
            throw new TaskExecutionException(e);
        }

        outputMap.put("id", inputMap.get("id")); 
        return outputMap;
    }
}
