/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
 */
package org.biobeans.modules.bistro.activity.onlineactivities.ncbi;

import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;
import org.biobeans.modules.bistro.activity.ActivityConfigurationException;
import org.biobeans.modules.bistro.activity.SimpleInputPort;
import org.biobeans.modules.bistro.data.DataContainer;
import org.biobeans.modules.bistro.data.DataContainerAdapter;
import org.biobeans.modules.bistro.io.TransmitterException;
import org.biobeans.modules.bistro.task.TaskExecutionException;

/**
 * This processor returns a INSD formatted nucleotide record
 *
 * @author mfortner
 *
 * @tavinput id The nucleotide accession.
 * @tavoutput outputText a INSD formatted nucleotide record
 */
public class NucleotideINSDSeqXMLActivity extends AbstractEFetchActivity<NucleotideINSDSeqXMLActivityConfigurationBean> {

    private static final Logger LOGGER = Logger.getLogger(NucleotideINSDSeqXMLActivity.class.getName());

    private NucleotideINSDSeqXMLActivityConfigurationBean configbean;
    
    public NucleotideINSDSeqXMLActivity() {
    }

    @Override
    public void configure(NucleotideINSDSeqXMLActivityConfigurationBean conf) throws ActivityConfigurationException {
        configbean = conf;
        configureEFetchActivity(configbean);
        addInputPort(new SimpleInputPort("id","text/plain",0,this));
    }

    @Override
    public NucleotideINSDSeqXMLActivityConfigurationBean getConfiguration() {
        return configbean;
    }

    @Override
    public Map<String, DataContainer> execute(Map<String, DataContainer> inputMap) throws TaskExecutionException {
        DataContainerAdapter inAdapter = new DataContainerAdapter(inputMap);
        this.id = inAdapter.getString("id");

        Map<String, String> transmitterMap = new HashMap<>();
        transmitterMap.put("id", this.id);
        transmitterMap.put("db", this.db);
        transmitterMap.put("rettype", this.rettype);
        transmitterMap.put("retmode", this.retmode);

        Map<String, DataContainer> outputMap;
        try {
            outputMap = this.transmit(transmitterMap);
        } catch (MalformedURLException e) {
            LOGGER.severe(this.url);
            throw new TaskExecutionException(e);
        } catch (TransmitterException e) {
            throw new TaskExecutionException(e);
        }

        return outputMap;
    }
}
