/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
 */
package org.biobeans.modules.bistro.activity.onlineactivities.ncbi;

import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.Map;
import org.biobeans.modules.bistro.activity.ActivityConfigurationException;
import org.biobeans.modules.bistro.activity.SimpleInputPort;
import org.biobeans.modules.bistro.data.DataContainer;
import org.biobeans.modules.bistro.data.DataContainerAdapter;
import org.biobeans.modules.bistro.io.TransmitterException;
import org.biobeans.modules.bistro.task.TaskExecutionException;

/**
 * This class is used to fetch pubmed articles in XML form. Use this worker only
 * if you already know the pubmed id.
 *
 * @author Mark
 *
 * @tavinput id A comma-delimited list of PubMed IDs.
 * @tavinput rettype Indicates how complete the returned records should be:
 * uilist, abstract, citation, medline, full (journals only)
 * @tavinput retmode Indicates the document format in which the results should
 * be returned. Select one of the following values: "xml", "html", "text",
 * "asn.1"
 *
 * @tavoutput outputText An XML representation of the PubMed article
 */
public class PubMedEFetchActivity extends AbstractEFetchActivity<PubMedEFetchActivityConfigurationBean> {

    public PubMedEFetchActivity() {
    }
    
    private PubMedEFetchActivityConfigurationBean configbean;

    @Override
    public void configure(PubMedEFetchActivityConfigurationBean conf) throws ActivityConfigurationException {
        configbean = conf;
        configureEFetchActivity(configbean);
        addInputPort(new SimpleInputPort("id","text/plain",0,this));
    }

    @Override
    public PubMedEFetchActivityConfigurationBean getConfiguration() {
        return configbean;
    }

    @Override
    public Map<String, DataContainer> execute(Map<String, DataContainer> inputMap) throws TaskExecutionException {
        DataContainerAdapter inAdapter = new DataContainerAdapter(inputMap);
        this.id = inAdapter.getString("id");

        String tempRetMode = inAdapter.getString("retmode");
        this.retmode = (tempRetMode != null && !tempRetMode.equals("")) ? tempRetMode : retmode;

        String tempRetType = inAdapter.getString("rettype");
        this.rettype = (tempRetType != null && !tempRetType.equals("")) ? tempRetType : rettype;

        Map<String, String> transmitterMap = new HashMap<>();
        transmitterMap.put("id", this.id);
        transmitterMap.put("db", this.db);
        transmitterMap.put("rettype", this.rettype);
        transmitterMap.put("retmode", this.retmode);
        
        Map<String, DataContainer> outputMap;
        try {
            outputMap = this.transmit(transmitterMap);
        } catch (MalformedURLException | TransmitterException e) {
            throw new TaskExecutionException(e);
        }

        outputMap.put("id", inputMap.get("id")); 
        return outputMap;
    }
}
