package org.biobeans.modules.bistro.activity.onlineactivities.ncbi;

import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.Map;
import org.biobeans.modules.bistro.activity.ActivityConfigurationException;
import org.biobeans.modules.bistro.activity.SimpleInputPort;
import org.biobeans.modules.bistro.data.DataContainer;
import org.biobeans.modules.bistro.data.DataContainerAdapter;
import org.biobeans.modules.bistro.io.TransmitterException;
import org.biobeans.modules.bistro.task.TaskExecutionException;

/**
 * This processor searches for articles in PubMed and returns their IDs in XML
 * format.
 *
 * @author Mark
 *
 * @tavinput term The search term.
 * @tavinput db The database to be searched (default=pubmed).
 * @tavinput field The optional database field (affl, auth, ecno, jour, iss,
 * mesh, majr, mhda, page, pdat, ptyp, si, subs, subh, tiab, word, titl, lang,
 * uid, fltr, vol)
 * @tavinput retstart (x= sequential number of the first record retrieved -
 * default=0 which will retrieve the first record)
 * @tavinput retmax (y= number of items retrieved)
 * @tavinput mindate The minimum date from which to begin the search.
 * @tavinput maxdate The maximu date from which to end the search.
 * @tavinput rettype count, uilist (default)
 *
 * @tavoutput outputText a PubMed ID list in XML format.
 *
 */
public class PubMedESearchActivity extends AbstractEFetchActivity<PubMedESearchActivityConfigurationBean> {

    protected String term = null;

    protected String field = null;

    private String mindate = null;

    private String maxdate = null;

    private PubMedESearchActivityConfigurationBean configbean;
    
    public PubMedESearchActivity() {
    }

    @Override
    public void configure(PubMedESearchActivityConfigurationBean conf) throws ActivityConfigurationException {
        configbean = conf;
        configureEFetchActivity(configbean);
        addInputPort(new SimpleInputPort("term","text/plain",0,this));
        addInputPort(new SimpleInputPort("field","text/plain",0,this));
        addInputPort(new SimpleInputPort("retstart","text/plain",0,this));
        addInputPort(new SimpleInputPort("retmax","text/plain",0,this));
        addInputPort(new SimpleInputPort("mindate","text/plain",0,this));
        addInputPort(new SimpleInputPort("maxdate","text/plain",0,this));
    }

    @Override
    public PubMedESearchActivityConfigurationBean getConfiguration() {
        return configbean;
    }

    @Override
    public Map<String, DataContainer> execute(Map<String, DataContainer> inputMap) throws TaskExecutionException {
        DataContainerAdapter inAdapter = new DataContainerAdapter(inputMap);
        this.term = inAdapter.getString("term");
        this.field = inAdapter.getString("field");
        this.retstart = inAdapter.getString("retstart");
        this.retmax = inAdapter.getString("retmax");
        this.mindate = inAdapter.getString("mindate");
        this.maxdate = inAdapter.getString("maxdate");

        Map<String, String> transmitterMap = new HashMap<>();
        transmitterMap.put("term", this.term);

        if (this.field != null) {
            transmitterMap.put("field", this.field);
        }

        if (this.retstart != null) {
            transmitterMap.put("retstart", this.retstart);
        }
        if (this.retmax != null) {
            transmitterMap.put("retmax", this.retmax);
        }

        if (this.mindate != null) {
            transmitterMap.put("mindate", this.mindate);
        }

        if (this.maxdate != null) {
            transmitterMap.put("maxdate", this.maxdate);
        }
        transmitterMap.put("db", this.db);

        if (this.rettype != null) {
            transmitterMap.put("rettype", this.rettype);
        }

        transmitterMap.put("retmode", this.retmode);
        
        Map results;
        try {
            results = transmit(transmitterMap);
        } catch (MalformedURLException | TransmitterException e) {
            throw new TaskExecutionException(e);
        }
        return results;
    }
}
