/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/
package org.biobeans.modules.bistro.onlineservices.ncbi;

import java.util.logging.Level;
import java.util.logging.Logger;
import org.biobeans.modules.bistro.activity.Activity;
import org.biobeans.modules.bistro.activity.ActivityConfigurationException;
import org.biobeans.modules.bistro.activity.SimpleConfigurationBean;
import org.biobeans.modules.bistro.activity.localactivities.ManualIDInputActivity;
import org.biobeans.modules.bistro.data.DataContainer;
import org.biobeans.modules.bistro.data.factory.DataContainerFactory;
import org.biobeans.modules.bistro.task.ActivityEvent;
import org.biobeans.modules.bistro.dataflow.DataListener;
import org.biobeans.modules.bistro.dataflow.DataPipe;
import org.biobeans.modules.bistro.dataflow.SimpleDataPipe;
import org.biobeans.modules.bistro.onlineservices.OnlineService;
import org.biobeans.modules.bistro.task.ActivityTask;
import org.biobeans.modules.bistro.task.Counter;

/**
 *
 * @author gwaldon
 */
public abstract class AbstractEFectchService extends OnlineService implements DataListener {
    
    private static final Logger LOGGER = Logger.getLogger(AbstractEFectchService.class.getName());

    DataPipe inputPipe;
    
    public AbstractEFectchService() {
        initialize();
    }

    private void initialize() {
        try {
            Activity inputID = new ManualIDInputActivity(getDialogLabel(),getDialogTitle());
            inputID.configure(new SimpleConfigurationBean(""));

            inputPipe = new SimpleDataPipe();
            inputID.getInputPort("fire").addPipe(inputPipe);

            DataPipe pipe = new SimpleDataPipe();
            inputID.getOutputPort("id").addPipe(pipe);
            Activity NCBIActivity = getActivity();
            NCBIActivity.getInputPort("id").addPipe(pipe);

            DataPipe outputPipe = new SimpleDataPipe();
            NCBIActivity.getOutputPort("outputText").addPipe(outputPipe);
            outputPipe.setSink(this);

            DataPipe idPipe = new SimpleDataPipe();
            NCBIActivity.getOutputPort("id").addPipe(idPipe);
            idPipe.setSink(this);
        } catch (ActivityConfigurationException ex) {
            LOGGER.log(Level.SEVERE, "COnfiguration exception in NCBI EFectch activity", ex);
        }
    }
    
    abstract String getDialogLabel();
    
    abstract String getDialogTitle();
    
    abstract protected Activity getActivity()throws ActivityConfigurationException;
    
    @Override
    public ActivityTask execute() {
        final ActivityTask at = new ActivityTask();
        
        Runnable run = new Runnable() {
            @Override
            public void run() {
                DataContainer dc = DataContainerFactory.create("");
                inputPipe.data(new ActivityEvent(at,"fire",dc));
            }
            
        };
        Counter.pour(run);
        return at;
    }
    
    @Override
    public void data(ActivityEvent event) {
        ActivityTask at = event.getTask();
        at.notifyActivity(event);
    }
}
