/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
*/
package org.biobeans.modules.bistro.onlineservices.ncbi;

import java.util.logging.Logger;
import org.biobeans.modules.bistro.activity.Activity;
import org.biobeans.modules.bistro.activity.ActivityConfigurationException;
import org.biobeans.modules.bistro.services.BioService;
import org.openide.util.NbBundle;
import org.biobeans.modules.bistro.activity.onlineactivities.ncbi.NucleotideGenBankActivity;
import org.biobeans.modules.bistro.activity.onlineactivities.ncbi.NucleotideGenBankActivityConfigurationBean;

/**
 *
 * @author George Waldon
 */
public class NucleotideGenBankService extends AbstractEFectchService {

    private static final Logger LOGGER = Logger.getLogger(NucleotideGenBankService.class.getName());

    private static NucleotideGenBankService service;

    public static BioService getDefault() {
        if (service == null) {
            service = new NucleotideGenBankService();
        }
        return service;
    }
    
    public NucleotideGenBankService() {
        setName(NbBundle.getMessage(NucleotideGenBankService.class, "LBL_NucleotideGenBankBioService"));
        setDescription(NbBundle.getMessage (NucleotideGenBankService.class, "LBL_fetchgenbank"));
    }

    @Override
    public String getID() {
        return "bistro:online:ncbi:nucleotidegenbank";
    }
    
    @Override
    String getDialogLabel() {
        return NbBundle.getMessage (NucleotideGenBankService.class, "LBL_fetchgenbank");
    }
    
    @Override
    String getDialogTitle() {
        return NbBundle.getMessage (NucleotideGenBankService.class, "LBL_enterseqid");
    }

    @Override
    protected Activity getActivity() throws ActivityConfigurationException {
        NucleotideGenBankActivity ac = new NucleotideGenBankActivity();
        NucleotideGenBankActivityConfigurationBean configga
                = new NucleotideGenBankActivityConfigurationBean();
        ac.configure(configga);
        addConfigurationBean(configga);
        return ac;
    }
}
