/*                Bistro development code
* 
* This code may be freely distributed and modified under the
* terms of the GNU Lesser General Public Licence.  This should
* be distributed with the code.  If you do not have a copy,
* see:
*
*     http://www.gnu.org/copyleft/lesser.html
* 
* Copyright for this code is held jointly by the individual
* authors.  These should be listed in @author doc comments.
* 
 */
package org.biobeans.modules.bistro.activity.onlineactivities.ncbi;

import java.io.StringReader;
import java.util.HashMap;
import java.util.Map;
import javax.xml.parsers.DocumentBuilderFactory;
import org.biobeans.modules.bistro.data.DataContainer;
import org.biobeans.modules.bistro.data.DataContainerAdapter;
import org.junit.Test;
import static org.junit.Assert.*;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/**
 *
 * @author George Waldon
 */
public class NucleotideGBActivityTest {
    
    private String interferon = "L25664";
    
    public NucleotideGBActivityTest() {
    }

    @Test
    public void testActivity() throws Exception {
        Map<String, DataContainer> inputMap1 = new HashMap<>();
        DataContainerAdapter inAdapter1 = new DataContainerAdapter(inputMap1);
        inAdapter1.putString("id", interferon);
        
        NucleotideGBSeqActivity ac = new NucleotideGBSeqActivity();
        NucleotideGBSeqActivityConfigurationBean config = 
                new NucleotideGBSeqActivityConfigurationBean();
        ac.configure(config);
        Map<String, DataContainer> outputMap1 = ac.execute(inputMap1);
        DataContainerAdapter outAdapter1 = new DataContainerAdapter(outputMap1);
        String results = outAdapter1.getString("outputText");
                
        assertNotNull("The results were null", results); 
        Element root = this.parseXml(results);
        testXmlNotEmpty(root);
    }
    
    /**
     * This method is used to parse the XML.
     * @param xml
     * @return
     * @throws Exception
     */
    public Element parseXml(String xml) throws Exception{
        Element root = null;
        
        // Create a builder factory
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setValidating(false);

        // Create the builder and parse the file
        Document doc = factory.newDocumentBuilder().parse(new InputSource(new StringReader(xml)));
        root = doc.getDocumentElement();
       
        return root;
    }
    
    /**
     * This method is meant to be called from within the testExecute method.
     * It verifies whether or not the XML that is returned by the worker is parseable
     * and valid.
     * @param xml
     * @return
     * @throws Exception
     */
    public void testXmlParsing(String xml) throws Exception{
        try {
            @SuppressWarnings("unused")
			Element root = parseXml(xml);
            
        }catch (Exception ex){
            ex.printStackTrace();
            fail("Not able to successfully parse xml" + xml);
        }
        
    }
    
    /**
     * This method is meant to be called from within the testExecute method.
     * It verifies whether or not the XML that is returned by the worker contains
     * any values.  By default all workers return this "<?xml version="1.0"?><searchResults></searchResults>"
     * If the searchResults element has children, then the results are not considered empty.
     * @param root
     * @throws Exception
     */
    public void testXmlNotEmpty(Element root) throws Exception{
        assertTrue("The root has no child nodes",root.hasChildNodes());
        NodeList nl = root.getChildNodes();
        int numNodes = nl.getLength();
        Node currNode = null;
        boolean hasElementNodes = false;
        for (int i=0; i < numNodes; i++){
            currNode = nl.item(i);
            if (currNode.getNodeType() == Node.ELEMENT_NODE){
                hasElementNodes = true;
                break;
            }
        }
        assertTrue("There were no element nodes", hasElementNodes);
    }
}